!tdo - enlever toute profondeur variable pour voir l'effet sur l'efficacite du code

!!
!! This module computes hydrologic processes on continental points.
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! @Version : $Revision: 1.35 $, $Date: 2007/06/12 20:02:37 $
!!
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/hydrol.f90,v 1.35 2007/06/12 20:02:37 ssipsl Exp $
!! IPSL (2006)
!!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!!                                                           
MODULE hydrol
  !
  !
  ! routines called : restput, restget
  !
  USE ioipsl
  !
  ! modules used :
  USE constantes
  USE constantes_soil
  USE constantes_veg
  USE sechiba_io

  ! for debug :
  USE grid

  IMPLICIT NONE

  ! public routines :
  ! hydrol
  PRIVATE
  PUBLIC :: hydrol_main,hydrol_clear 

  !
  ! variables used inside hydrol module : declaration and initialisation
  !
  LOGICAL, SAVE                                     :: l_first_hydrol=.TRUE. !! Initialisation has to be done one time
  !
  LOGICAL, SAVE                                     :: check_cwrr=.FALSE. !! The check the water balance
  LOGICAL, SAVE                                     :: doponds=.FALSE. !! Reinfiltration param.

  CHARACTER(LEN=80) , SAVE                          :: file_ext         !! Extention for I/O filename
  CHARACTER(LEN=80) , SAVE                          :: var_name         !! To store variables names for I/O
  REAL(r_std), PARAMETER                             :: drain_rest_cste = 15.0  !! time constant in days to return to free drainage after return flow
  REAL(r_std), PARAMETER                             :: allowed_err =  2.0E-8_r_std
  ! one dimension array allocated, computed, saved and got in hydrol module
  ! Values per soil type
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: nvan
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: avan
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mcr
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mcs                   !! Saturation
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: ks
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:) :: ds
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: pcent
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: free_drain_max
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mcf                   !! Field capacity / Capacit au champ
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mcw                   !! Wilting point / Point de fltrissement
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mc_awet
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: mc_adry
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: psis

  ! Values per grid point
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_water_beg    !! Total amount of water at start of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_water_end    !! Total amount of water at end of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_flux         !! Total water flux
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watveg_beg   !! Total amount of water on vegetation at start of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watveg_end   !! Total amount of water on vegetation at end of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watsoil_beg  !! Total amount of water in the soil at start of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watsoil_end  !! Total amount of water in the soil at end of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: snow_beg         !! Total amount of snow at start of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: snow_end         !! Total amount of snow at end of time step
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: delsoilmoist     !! Change in soil moisture
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: delintercept     !! Change in interception storage
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: delswe           !! Change in SWE^Q
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: swi              !! Soil Wetness Index

  ! array allocated, computed, saved and got in hydrol module
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: mask_veget           !! zero/one when veget fraction is zero/higher
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: mask_soiltile        !! zero/one where soil fraction is zero/higher 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: humrelv          !! humrel for each soil type
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: vegstressv       !! vegstress for each soil type  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:,:):: us               !! relative humidity 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: precisol         !! Eau tombee sur le sol
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: precisol_ns      !! Eau tombee sur le sol par type de sol
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: ae_ns            !! Evaporation du sol nu par type de sol
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: evap_bare_lim_ns !! limitation of bare soil evaporation on each soil column (used to deconvoluate vevapnu)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: free_drain_coef !! Coefficient for free drainage at bottom
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: frac_bare_ns   !! evaporating bare soil fraction per tile
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_frac_bare   !! total evaporating bare soil fraction 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: rootsink         !! stress racinaire par niveau et type de sol
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: subsnowveg       !! Sublimation of snow on vegetation
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: subsnownobio     !! Sublimation of snow on other surface types (ice, lakes, ...)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: snowmelt         !! Quantite de neige fondue
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: icemelt          !! Quantite de glace fondue
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: subsinksoil      !! Excess of sublimation as a sink for the soil
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: vegtot           !! Total  vegetation
  ! The last vegetation map which was used to distribute the reservoirs
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: resdist          !! Distribution of reservoirs
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: mx_eau_var

  ! arrays used by cwrr scheme
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: nroot            !! nvm * nstm * nslm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: kfact_root       !! kjpindex * nslm * nstm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: kfact            !! nslm * nscm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: zz               !! nslm+1 * nstm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: dz               !! nslm+1 * nstm

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: mc_lin             !! imin:imax * nscm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: k_lin              !! imin:imax * nslm * nscm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: d_lin              !! imin:imax * nslm * nscm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: a_lin              !! imin:imax * nslm * nscm
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: b_lin              !! imin:imax * nslm * nscm

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: humtot      !! (:)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: flux        !! (:)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION (:)         :: resolv      !! (:)


!! linarization coefficients of hydraulic conductivity K
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: k           !! (:,nslm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: a           !! (:,nslm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: b           !!
!! linarization coefficients of hydraulic diffusivity D
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: d           !!

!! matrix coefficients
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: e           !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: f           !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: g1          !!


  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: ep          !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: fp          !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: gp          !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: rhs         !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: srhs        !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: gam         !!

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: water2infilt     !! Water to be infiltrated
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc         !! (:,nstm) Total moisture content (mm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmcr        !! (nstm) Total moisture constent at residual (mm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmcs        !! (nstm) Total moisture constent at saturation (mm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter       !! (:,nstm) Total moisture in the litter by soil type
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)      :: tmc_litt_mea !! Total moisture in the litter over the grid

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_wilt  !! (:,nstm) Moisture of litter at wilt pt
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_field !! (:,nstm) Moisture of litter at field cap.
!!! A CHANGER DANS TOUT HYDROL: tmc_litter_res et sat ne devraient pas dependre de ji - tdo
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_res !! (:,nstm) Moisture of litter at residual moisture.
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_sat   !! (:,nstm) Moisture of litter at saturatiion
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_awet   !! (:,nstm) Moisture of litter at mc_awet
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tmc_litter_adry   !! (:,nstm) Moisture of litter at mc_dry
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: tmc_litt_wet_mea !! Total moisture in the litter over the grid below which albedo is fixed
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: tmc_litt_dry_mea !! Total moisture in the litter over the grid above which albedo is fixed

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: v1          !! (:)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: vB          !! (:)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: qflux00     !! flux at the top of the soil column

  !! par type de sol :
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: ru_ns       !! ruissellement
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: dr_ns       !! drainage
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: tr_ns       !! transpiration 

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: cvs_over_veg  !! (:,nvm,nstm) old value of corr_veg_soil/veget kept from diag to next split
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: corr_veg_soil !! (:,nvm,nstm) percentage of each veg. type on each soil of each grid point
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: mc          !! (:,nslm,nstm) m x m
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: soilmoist     !! (:,nslm)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: soil_wet    !! soil wetness
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: soil_wet_litter !! soil wetness of the litter
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: qflux       !! fluxes between the soil layers

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: tmat        !!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: stmat        !!

  LOGICAL, SAVE                                     :: interpol_diag=.FALSE.

  INTEGER(i_std), SAVE                         :: kjit_             !! Time step number 

CONTAINS

  !! 
  !! Main routine for *hydrol* module
  !! - called only one time for initialisation
  !! - called every time step
  !! - called one more time at last time step for writing _restart_ file
  !!
  !! Algorithm:
  !! - call hydrol_snow for snow process (including age of snow)
  !! - call hydrol_canop for canopy process
  !! - call hydrol_soil for bare soil process
  !!
  !! @call hydrol_snow
  !! @call hydrol_canop
  !! @call hydrol_soil
  !!
  SUBROUTINE hydrol_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
       & index, indexveg, indexsoil, indexlayer, &
       & temp_sol_new, floodout, runoff, drainage, frac_nobio, totfrac_nobio, vevapwet, veget, frac_bare, njsc, &
       & qsintmax, qsintveg, vevapnu, vevapsno, vevapflo, snow, snow_age, snow_nobio, snow_nobio_age,  &
       & tot_melt, transpir, precip_rain, precip_snow, returnflow, reinfiltration, irrigation, &
       & humrel, vegstress, drysoil_frac, evapot, evapot_penm, evap_bare_lim, flood_frac, flood_res, &
       & shumdiag, k_litt, litterhumdiag, soilcap, soiltile, reinf_slope, rest_id, hist_id, hist2_id)   

    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                         :: kjit             !! Time step number 
    INTEGER(i_std), INTENT(in)                         :: kjpindex         !! Domain size
    INTEGER(i_std),INTENT (in)                         :: rest_id,hist_id  !! _Restart_ file and _history_ file identifier
    INTEGER(i_std),INTENT (in)                         :: hist2_id         !! _history_ file 2 identifier
    REAL(r_std), INTENT (in)                           :: dtradia          !! Time step in seconds
    LOGICAL, INTENT(in)                               :: ldrestart_read   !! Logical for _restart_ file to read
    LOGICAL, INTENT(in)                               :: ldrestart_write  !! Logical for _restart_ file to write
    ! input fields
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: index            !! Indeces of the points on the map
    INTEGER(i_std),DIMENSION (kjpindex*nvm), INTENT (in):: indexveg        !! Indeces of the points on the 3D map for veg
    INTEGER(i_std),DIMENSION (kjpindex*nstm), INTENT (in):: indexsoil      !! Indeces of the points on the 3D map for soil
    INTEGER(i_std),DIMENSION (kjpindex*nslm), INTENT (in):: indexlayer     !! Indeces of the points on the 3D map for soil layers
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: precip_rain      !! Rain precipitation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: precip_snow      !! Snow precipitation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: returnflow       !! Routed water which comes back into the soil (from the bottom)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: reinfiltration   !! Routed water which comes back into the soil (at the top)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: irrigation       !! Water from irrigation returning to soil moisture  
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: temp_sol_new     !! New soil temperature

    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: njsc            !! Indeces of the soiltype 
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (in) :: frac_nobio     !! Fraction of ice, lakes, ...
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: totfrac_nobio    !! Total fraction of ice+lakes+...
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: soilcap          !! Soil capacity
    REAL(r_std),DIMENSION (kjpindex,nstm), INTENT (in) :: soiltile         !! Map of soil types
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: vevapwet         !! Interception loss
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: frac_bare        !! Fraction of bare soil in each vegetation type           
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget            !! Fraction of vegetation type           
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintmax         !! Maximum water on vegetation for interception
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: transpir         !! Transpiration
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: reinf_slope            !! slope coef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: evapot           !! Soil Potential Evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: evapot_penm      !! Soil Potential Evaporation Correction
    ! modified fields
    REAL(r_std),DIMENSION (kjpindex), INTENT(out)      :: evap_bare_lim    !!
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: vevapnu          !! Bare soil evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: vevapsno         !! Snow evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: vevapflo         !! Snow evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: flood_res        !! flood reservoir estimate
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: flood_frac       !! flood fraction
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: snow             !! Snow mass [Kg/m^2]
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)   :: snow_age         !! Snow age
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (inout) :: snow_nobio     !! Water balance on ice, lakes, .. [Kg/m^2]
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (inout) :: snow_nobio_age !! Snow age on ice, lakes, ...
    !! We consider that any water on the ice is snow and we only peforme a water balance to have consistency.
    !! The water balance is limite to + or - 10^6 so that accumulation is not endless
    ! output fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: floodout         !! flux out of floodplains
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: runoff           !! Complete runoff
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: drainage         !! Drainage
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out) :: humrel           !! Relative humidity
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out) :: vegstress        !! Veg. moisture stress (only for vegetation growth)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: drysoil_frac     !! function of litter wetness
    REAL(r_std),DIMENSION (kjpindex,nbdl), INTENT (out):: shumdiag         !! relative soil moisture
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: k_litt           !! litter approximate conductivity
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: litterhumdiag    !! litter humidity
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: tot_melt         !! Total melt    
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out) :: qsintveg         !! Water on vegetation due to interception

    !
    ! local declaration
    !
    INTEGER(i_std)                                    :: jst, jsl
    REAL(r_std),DIMENSION (kjpindex)                   :: soilwet          !! A temporary diagnostic of soil wetness
    REAL(r_std),DIMENSION (kjpindex)                   :: snowdepth        !! Depth of snow layer
    REAL(r_std),DIMENSION (kjpindex)                   :: njsc_tmp         !! Temporary REAL value for njsc to write it

    kjit_=kjit
    !
    ! do initialisation
    !
    IF (l_first_hydrol) THEN

       IF (long_print) WRITE (numout,*) ' l_first_hydrol : call hydrol_init '

       CALL hydrol_init (kjit, ldrestart_read, kjpindex, index, rest_id, veget, soiltile, humrel,&
            & vegstress, snow, snow_age, snow_nobio, snow_nobio_age, qsintveg) 
       CALL hydrol_var_init (kjpindex, veget, soiltile, njsc, mx_eau_var, shumdiag, k_litt, &
            & litterhumdiag, drysoil_frac, evap_bare_lim) 
       !
       ! If we check the water balance we first save the total amount of water
       !
       IF (check_waterbal) THEN
          CALL hydrol_waterbal(kjpindex, index, .TRUE., dtradia, veget, &
               & totfrac_nobio, qsintveg, snow, snow_nobio,&
               & precip_rain, precip_snow, returnflow, reinfiltration, irrigation, tot_melt, &
               & vevapwet, transpir, vevapnu, vevapsno, vevapflo, floodout, runoff,drainage)
       ENDIF
       !
       IF (almaoutput) THEN
          CALL hydrol_alma(kjpindex, index, .TRUE., qsintveg, snow, snow_nobio, soilwet)
       ENDIF

       RETURN

    ENDIF

    !
    ! prepares restart file for the next simulation
    !
    IF (ldrestart_write) THEN

       IF (long_print) WRITE (numout,*) ' we have to complete restart file with HYDROLOGIC variables '

       DO jst=1,nstm
          ! var_name= "mc_1" ... "mc_3"
          WRITE (var_name,"('moistc_',i1)") jst
          CALL restput_p(rest_id, var_name, nbp_glo,  nslm, 1, kjit, mc(:,:,jst), 'scatter',  nbp_glo, index_g)
       END DO
       !
       DO jst=1,nstm
          DO jsl=1,nslm
             ! var_name= "us_1_01" ... "us_3_11"
             WRITE (var_name,"('us_',i1,'_',i2.2)") jst,jsl
             CALL restput_p(rest_id, var_name, nbp_glo,nvm, 1,kjit,us(:,:,jst,jsl),'scatter',nbp_glo,index_g)
          END DO
       END DO
       !
       var_name= 'free_drain_coef'  
       CALL restput_p(rest_id, var_name, nbp_glo,   nstm, 1, kjit,  free_drain_coef, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'water2infilt'  
       CALL restput_p(rest_id, var_name, nbp_glo,   nstm, 1, kjit,  water2infilt, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'ae_ns'  
       CALL restput_p(rest_id, var_name, nbp_glo,   nstm, 1, kjit,  ae_ns, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'vegstress'
       CALL restput_p(rest_id, var_name, nbp_glo,   nvm, 1, kjit,  vegstress, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'snow'    
       CALL restput_p(rest_id, var_name, nbp_glo,   1, 1, kjit,  snow, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'snow_age'
       CALL restput_p(rest_id, var_name, nbp_glo,   1, 1, kjit,  snow_age, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'snow_nobio'    
       CALL restput_p(rest_id, var_name, nbp_glo,   nnobio, 1, kjit,  snow_nobio, 'scatter', nbp_glo, index_g)
       !
       var_name= 'snow_nobio_age'
       CALL restput_p(rest_id, var_name, nbp_glo,   nnobio, 1, kjit,  snow_nobio_age, 'scatter', nbp_glo, index_g)
       !
       var_name= 'qsintveg'
       CALL restput_p(rest_id, var_name, nbp_glo, nvm, 1, kjit,  qsintveg, 'scatter',  nbp_glo, index_g)
       !
       var_name= 'resdist'
       CALL restput_p(rest_id, var_name, nbp_glo, nvm, 1, kjit,  resdist, 'scatter',  nbp_glo, index_g)       
       !
       DO jst=1,nstm
          ! var_name= "cvs_over_veg_1" ... "cvs_over_veg_3"
          WRITE (var_name,"('cvs_over_veg_',i1)") jst
          CALL restput_p(rest_id, var_name, nbp_glo,  nvm, 1, kjit, cvs_over_veg(:,:,jst), 'scatter',  nbp_glo, index_g)
       END DO
       !
!!$       DO jst=1,nstm
!!$          ! var_name= "humrelv_1" ... "humrelv_3"
!!$          WRITE (var_name,"('humrelv_',i1)") jst
!!$          CALL restput_p(rest_id, var_name, nbp_glo,  nvm, 1, kjit, humrelv(:,:,jst), 'scatter',  nbp_glo, index_g)
!!$       END DO
       !
       RETURN
       !
    END IF

    !
    ! shared time step
    !
    IF (long_print) WRITE (numout,*) 'hydrol pas de temps = ',dtradia

    ! 
    ! computes snow
    !

    CALL hydrol_snow(kjpindex, dtradia, precip_rain, precip_snow, temp_sol_new, soilcap, &
         & frac_nobio, totfrac_nobio, vevapnu, vevapsno, snow, snow_age, snow_nobio, snow_nobio_age, &
         & tot_melt, snowdepth)

    !
    ! computes canopy
    !
    !
    CALL hydrol_vegupd(kjpindex, veget, frac_bare, soiltile, qsintveg,resdist)
    !

    CALL hydrol_canop(kjpindex, precip_rain, vevapwet, veget, qsintmax, qsintveg,precisol,tot_melt)

    !
    ! computes surface reservoir
    !

    CALL hydrol_flood(kjpindex, dtradia, vevapflo, flood_frac, flood_res, floodout)

    !
    ! computes hydro_soil
    !

    CALL hydrol_soil(kjpindex, dtradia, veget, soiltile, njsc, reinf_slope,  &
         & transpir, vevapnu, evapot, evapot_penm, runoff, drainage, returnflow, reinfiltration, irrigation, &
         & tot_melt,evap_bare_lim, shumdiag, k_litt, litterhumdiag, humrel, vegstress, drysoil_frac)
    !
    ! If we check the water balance we end with the comparison of total water change and fluxes
    !
    IF (check_waterbal) THEN
       CALL hydrol_waterbal(kjpindex, index, .FALSE., dtradia, veget, totfrac_nobio, &
            & qsintveg, snow, snow_nobio, precip_rain, precip_snow, returnflow, reinfiltration, &
            & irrigation, tot_melt, vevapwet, transpir, vevapnu, vevapsno, vevapflo, floodout, runoff, drainage)
    ENDIF
    !
    ! If we use the ALMA standards
    !
    IF (almaoutput) THEN
       CALL hydrol_alma(kjpindex, index, .FALSE., qsintveg, snow, snow_nobio, soilwet)
    ENDIF

    !
    IF ( .NOT. almaoutput ) THEN
       DO jst=1,nstm
          ! var_name= "mc_1" ... "mc_3"
          WRITE (var_name,"('moistc_',i1)") jst
          CALL histwrite(hist_id, trim(var_name), kjit,mc(:,:,jst), kjpindex*nslm, indexlayer)

          ! var_name= "kfactroot_1" ... "kfactroot_3"
          WRITE (var_name,"('kfactroot_',i1)") jst
          CALL histwrite(hist_id, trim(var_name), kjit, kfact_root(:,:,jst), kjpindex*nslm, indexlayer)

! var_name= "vegetsoil_1" ... "vegetsoil_3"
!          WRITE (var_name,"('vegetsoil_',i1)") jst
!         CALL histwrite(hist_id, trim(var_name), kjit,corr_veg_soil(:,:,jst), kjpindex*nvm, indexveg)
       ENDDO
       CALL histwrite(hist_id, 'evapnu_soil', kjit, ae_ns, kjpindex*nstm, indexsoil)
       CALL histwrite(hist_id, 'drainage_soil', kjit, dr_ns, kjpindex*nstm, indexsoil)
       CALL histwrite(hist_id, 'transpir_soil', kjit, tr_ns, kjpindex*nstm, indexsoil)
       CALL histwrite(hist_id, 'runoff_soil', kjit, ru_ns, kjpindex*nstm, indexsoil)
       CALL histwrite(hist_id, 'humtot_soil', kjit, tmc, kjpindex*nstm, indexsoil)
       CALL histwrite(hist_id, 'humtot', kjit, humtot, kjpindex, index)
       njsc_tmp(:)=njsc(:)
       CALL histwrite(hist_id, 'njsc', kjit, njsc_tmp, kjpindex, index)
       CALL histwrite(hist_id, 'humrel',   kjit, humrel,   kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'drainage', kjit, drainage, kjpindex, index)
       CALL histwrite(hist_id, 'runoff', kjit, runoff, kjpindex, index)
       CALL histwrite(hist_id, 'floodout', kjit, floodout, kjpindex, index)
       CALL histwrite(hist_id, 'precisol', kjit, precisol, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'rain', kjit, precip_rain, kjpindex, index)
       CALL histwrite(hist_id, 'snowf', kjit, precip_snow, kjpindex, index)
       CALL histwrite(hist_id, 'qsintmax', kjit, qsintmax, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'qsintveg', kjit, qsintveg, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'SWI', kjit, swi, kjpindex, index)
       IF ( hist2_id > 0 ) THEN
          DO jst=1,nstm
             ! var_name= "mc_1" ... "mc_3"
             WRITE (var_name,"('moistc_',i1)") jst
             CALL histwrite(hist2_id, trim(var_name), kjit,mc(:,:,jst), kjpindex*nslm, indexlayer)

             ! var_name= "kfactroot_1" ... "kfactroot_3"
             WRITE (var_name,"('kfactroot_',i1)") jst
             CALL histwrite(hist2_id, trim(var_name), kjit, kfact_root(:,:,jst), kjpindex*nslm, indexlayer)

! var_name= "vegetsoil_1" ... "vegetsoil_3"
!             WRITE (var_name,"('vegetsoil_',i1)") jst
!            CALL histwrite(hist2_id, trim(var_name), kjit,corr_veg_soil(:,:,jst), kjpindex*nvm, indexveg)
          ENDDO
          CALL histwrite(hist2_id, 'evapnu_soil', kjit, ae_ns, kjpindex*nstm, indexsoil)
          CALL histwrite(hist2_id, 'drainage_soil', kjit, dr_ns, kjpindex*nstm, indexsoil)
          CALL histwrite(hist2_id, 'transpir_soil', kjit, tr_ns, kjpindex*nstm, indexsoil)
          CALL histwrite(hist2_id, 'runoff_soil', kjit, ru_ns, kjpindex*nstm, indexsoil)
          CALL histwrite(hist2_id, 'humtot_soil', kjit, tmc, kjpindex*nstm, indexsoil)
          CALL histwrite(hist2_id, 'humtot', kjit, humtot, kjpindex, index)
          njsc_tmp(:)=njsc(:)
          CALL histwrite(hist2_id, 'njsc', kjit, njsc_tmp, kjpindex, index)
          CALL histwrite(hist2_id, 'humrel',   kjit, humrel,   kjpindex*nvm, indexveg)
          CALL histwrite(hist2_id, 'drainage', kjit, drainage, kjpindex, index)
          CALL histwrite(hist2_id, 'runoff', kjit, runoff, kjpindex, index)
          CALL histwrite(hist2_id, 'floodout', kjit, floodout, kjpindex, index)
          CALL histwrite(hist2_id, 'precisol', kjit, precisol, kjpindex*nvm, indexveg)
          CALL histwrite(hist2_id, 'rain', kjit, precip_rain, kjpindex, index)
          CALL histwrite(hist2_id, 'snowf', kjit, precip_snow, kjpindex, index)
          CALL histwrite(hist2_id, 'qsintmax', kjit, qsintmax, kjpindex*nvm, indexveg)
          CALL histwrite(hist2_id, 'qsintveg', kjit, qsintveg, kjpindex*nvm, indexveg)
          CALL histwrite(hist2_id, 'SWI', kjit, swi, kjpindex, index) 
          !
          IF (check_waterbal) THEN
             CALL histwrite(hist2_id, 'TotWater', kjit, tot_water_end, kjpindex, index)
             CALL histwrite(hist2_id, 'TotWaterFlux', kjit, tot_flux, kjpindex, index)
          ENDIF
      ENDIF
    ELSE
       CALL histwrite(hist_id, 'Snowf', kjit, precip_snow, kjpindex, index)
       CALL histwrite(hist_id, 'Rainf', kjit, precip_rain, kjpindex, index)
       CALL histwrite(hist_id, 'Qs', kjit, runoff, kjpindex, index)
       CALL histwrite(hist_id, 'Qsb', kjit, drainage, kjpindex, index)
       CALL histwrite(hist_id, 'Qsm', kjit, tot_melt, kjpindex, index)
       CALL histwrite(hist_id, 'DelSoilMoist', kjit, delsoilmoist, kjpindex, index)
       CALL histwrite(hist_id, 'DelIntercept', kjit, delintercept, kjpindex, index)
       !
       CALL histwrite(hist_id, 'SoilMoist', kjit, soilmoist, kjpindex*nslm, indexlayer)
       CALL histwrite(hist_id, 'SoilWet', kjit, soilwet, kjpindex, index)
!       CALL histwrite(hist_id, 'SWI', kjit, swi, kjpindex, index)
       !
       CALL histwrite(hist_id, 'RootMoist', kjit, tot_watsoil_end, kjpindex, index)
       CALL histwrite(hist_id, 'SubSnow', kjit, vevapsno, kjpindex, index)
       !
       CALL histwrite(hist_id, 'SnowDepth', kjit, snowdepth, kjpindex, index)
       !
       IF ( hist2_id > 0 ) THEN
          CALL histwrite(hist2_id, 'Snowf', kjit, precip_snow, kjpindex, index)
          CALL histwrite(hist2_id, 'Rainf', kjit, precip_rain, kjpindex, index)
          CALL histwrite(hist2_id, 'Qs', kjit, runoff, kjpindex, index)
          CALL histwrite(hist2_id, 'Qsb', kjit, drainage, kjpindex, index)
          CALL histwrite(hist2_id, 'Qsm', kjit, tot_melt, kjpindex, index)
          CALL histwrite(hist2_id, 'DelSoilMoist', kjit, delsoilmoist, kjpindex, index)
          CALL histwrite(hist2_id, 'DelIntercept', kjit, delintercept, kjpindex, index)
          !
          CALL histwrite(hist2_id, 'SoilMoist', kjit, soilmoist, kjpindex*nslm, indexlayer)
          CALL histwrite(hist2_id, 'SoilWet', kjit, soilwet, kjpindex, index)
          CALL histwrite(hist2_id, 'SWI', kjit, swi, kjpindex, index)
          !
          CALL histwrite(hist2_id, 'RootMoist', kjit, tot_watsoil_end, kjpindex, index)
          CALL histwrite(hist2_id, 'SubSnow', kjit, vevapsno, kjpindex, index)
          !
          CALL histwrite(hist2_id, 'SnowDepth', kjit, snowdepth, kjpindex, index)
       ENDIF
    ENDIF

    IF (long_print) WRITE (numout,*) ' hydrol_main Done '

  END SUBROUTINE hydrol_main

  !! Algorithm:
  !! - dynamic allocation for local array 
  !! - _restart_ file reading for HYDROLOGIC variables
  !!
  SUBROUTINE hydrol_init(kjit, ldrestart_read, kjpindex, index, rest_id, veget, soiltile, humrel,&
       &  vegstress, snow, snow_age, snow_nobio, snow_nobio_age, qsintveg)

    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT (in)                         :: kjit               !! Time step number 
    LOGICAL,INTENT (in)                                :: ldrestart_read     !! Logical for _restart_ file to read
    INTEGER(i_std), INTENT (in)                         :: kjpindex           !! Domain size
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)    :: index              !! Indeces of the points on the map
    INTEGER(i_std), INTENT (in)                         :: rest_id            !! _Restart_ file identifier 
    ! input fields
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: veget              !! Carte de vegetation
    REAL(r_std),DIMENSION (kjpindex,nstm), INTENT (in)  :: soiltile         !! Map of soil types
    ! output fields
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: humrel             !! Stress hydrique, relative humidity
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out) :: vegstress     !! Veg. moisture stress (only for vegetation growth)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: snow               !! Snow mass [Kg/m^2]
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: snow_age           !! Snow age
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (out) :: snow_nobio       !! Snow on ice, lakes, ...
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (out) :: snow_nobio_age   !! Snow age on ice, lakes, ...
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: qsintveg           !! Water on vegetation due to interception

    ! local declaration
    INTEGER(i_std)                                     :: ier, ierror, ipdt
    INTEGER(i_std)                                     :: ji, jv, jst, jsl, ik, jsc
    REAL(r_std),DIMENSION (kjpindex,nvm,nstm) :: humrelv_temp                 !! initialization for humrelv

    ! initialisation
    IF (l_first_hydrol) THEN 
       l_first_hydrol=.FALSE.
    ELSE 
       WRITE (numout,*) ' l_first_hydrol false . we stop '
       STOP 'hydrol_init'
    ENDIF

    !
    ! Some initializations
    !
    !
    !Config Key  = CHECK_CWRR
    !Config Desc = Should we check detailed CWRR water balance ?
    !Config Def  = FALSE
    !Config Help = This parameters allows the user to check
    !Config        the detailed water balance in each time step
    !Config        of CWRR.
    !
    check_cwrr = .FALSE.
    CALL getin_p('CHECK_CWRR', check_cwrr)
    !
    !Config Key  = DOPONDS
    !Config Desc = Should we include ponds 
    !Config Def  = FALSE
    !Config Help = This parameters allows the user to ask the model
    !Config        to take into account the ponds and return 
    !Config        the water into the soil moisture. If this is 
    !Config        activated, then there is no reinfiltration 
    !Config        computed inside the hydrol module.
    !
    doponds = .FALSE.
    CALL getin_p('DO_PONDS', doponds)
    

    ! make dynamic allocation with good dimension

    ! one dimension array allocation with possible restart value

    ALLOCATE (nvan(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (avan(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mcr(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mcs(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (ks(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (ds(nslm,nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nslm*nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (pcent(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (free_drain_max(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mcf(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mcw(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (mc_awet(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mc_adry(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF
       
    ALLOCATE (psis(nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need nscm words = ',nscm
       STOP 'hydrol_init'
    END IF
    
    SELECTCASE (nscm)
    CASE (3)
              
       nvan = nvan_fao       
       avan = avan_fao
       mcr = mcr_fao
       mcs = mcs_fao
       ks = ks_fao
       ds(:,:)=zero
       pcent = pcent_fao
       free_drain_max = free_drain_max_fao
       mcf = mcf_fao
       mcw = mcw_fao
       mc_awet = mc_awet_fao
       mc_adry = mc_adry_fao
       psis = psis_fao
       
    CASE (5)
              
       DO jsc =1, nscm 
          nvan(jsc) = nvan_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          avan(jsc) = avan_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mcr(jsc) = mcr_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mcs(jsc) = mcs_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          ks(jsc) = ks_fao(CEILING(jsc*un/deux))
       ENDDO
       ds(:,:)=zero
       DO jsc =1, nscm 
          pcent(jsc) = pcent_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          free_drain_max(jsc) = free_drain_max_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mcf(jsc) = mcf_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mcw(jsc) = mcw_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mc_awet(jsc) = mc_awet_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          mc_adry(jsc) = mc_adry_fao(CEILING(jsc*un/deux))
       ENDDO
       DO jsc =1, nscm 
          psis(jsc) = psis_fao(CEILING(jsc*un/deux))
       ENDDO
       
    CASE (12)
       
       nvan = nvan_usda
       avan = avan_usda
       mcr = mcr_usda
       mcs = mcs_usda
       ks = ks_usda
       ds(:,:)=zero
       pcent = pcent_usda
       free_drain_max = free_drain_max_usda
       mcf = mcf_usda
       mcw = mcw_usda
       mc_awet = mc_awet_usda
       mc_adry = mc_adry_usda
       psis = psis_usda
       
    CASE DEFAULT
       WRITE (numout,*) 'Unsupported soil type classification. Choose between zobler, fao and usda according to the map'
       STOP 'hydrol_init'
    ENDSELECT


    ALLOCATE (mask_veget(kjpindex,nvm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in mask_veget allocation. We stop. We need kjpindex words = ',kjpindex*nvm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mask_soiltile(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in mask_soiltile allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (humrelv(kjpindex,nvm,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in humrelv allocation. We stop. We need kjpindex words = ',kjpindex*nvm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (vegstressv(kjpindex,nvm,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vegstressv allocation. We stop. We need kjpindex words = ',kjpindex*nvm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (us(kjpindex,nvm,nstm,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in us allocation. We stop. We need kjpindex words = ',kjpindex*nvm*nstm*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (precisol(kjpindex,nvm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in precisol allocation. We stop. We need kjpindex words = ',kjpindex*nvm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (precisol_ns(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in precisol_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (free_drain_coef(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in free_drain_coef allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (frac_bare_ns(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in frac_bare_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tot_frac_bare(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_frac_bare allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (water2infilt(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in water2infilt allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (ae_ns(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in ae_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (evap_bare_lim_ns(kjpindex,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in evap_bare_lim_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (rootsink(kjpindex,nslm,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rootsink allocation. We stop. We need kjpindex words = ',kjpindex*nslm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (subsnowveg(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in subsnowveg allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (subsnownobio(kjpindex,nnobio),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in subsnownobio allocation. We stop. We need kjpindex words = ',kjpindex*nnobio
       STOP 'hydrol_init'
    END IF

    ALLOCATE (snowmelt(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snowmelt allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (icemelt(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in icemelt allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (subsinksoil(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in subsinksoil allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mx_eau_var(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in mx_eau_var allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (vegtot(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vegtot allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (resdist(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in resdist allocation. We stop. We need kjpindex words = ',kjpindex*nvm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (humtot(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in humtot allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (flux(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in flux allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (resolv(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in resolv allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (k(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in k allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (a(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in a allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (b(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in b allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (d(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in d allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (e(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in e allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (f(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in f allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (g1(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in g1 allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (ep(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in ep allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (fp(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in fp allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (gp(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in gp allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (rhs(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rhs allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (srhs(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in srhs allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (gam(kjpindex,nslm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in gam allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmcs(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmcs allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmcr(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmcr allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litt_mea(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litt_mea allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_res(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_res allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_wilt(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_wilt allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_field(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_field allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_sat(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_sat allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_awet(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_awet allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litter_adry(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litter_adry allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litt_wet_mea(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litt_wet_mea allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmc_litt_dry_mea(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmc_litt_dry_mea allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF

    ALLOCATE (v1(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in v1 allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

!    ALLOCATE (vB(kjpindex,nstm),stat=ier)
!    IF (ier.NE.0) THEN
!       WRITE (numout,*) ' error in vB allocation. We stop. We need kjpindex words = ',kjpindex*nstm
!       STOP 'hydrol_init'
!    END IF

    ALLOCATE (qflux00(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qflux00 allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (ru_ns(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in ru_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (dr_ns(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in dr_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tr_ns(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tr_ns allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (cvs_over_veg(kjpindex,nvm,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in cvs_over_veg allocation. We stop. We need kjpindex words = ',kjpindex*nvm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (corr_veg_soil(kjpindex,nvm,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in corr_veg_soil allocation. We stop. We need kjpindex words = ',kjpindex*nvm*nstm
       STOP 'hydrol_init'
    END IF


    ALLOCATE (mc(kjpindex,nslm,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in mc allocation. We stop. We need kjpindex words = ',kjpindex*nslm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (soilmoist(kjpindex,nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soilmoist allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (soil_wet(kjpindex,nslm,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soil_wet allocation. We stop. We need kjpindex words = ',kjpindex*nslm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (soil_wet_litter(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soil_wet allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (qflux(kjpindex,nslm,nstm),stat=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qflux allocation. We stop. We need kjpindex words = ',kjpindex*nslm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (tmat(kjpindex,nslm,3),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tmat allocation. We stop. We need kjpindex words = ',kjpindex*nslm*trois
       STOP 'hydrol_init'
    END IF

    ALLOCATE (stmat(kjpindex,nslm,3),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in stmat allocation. We stop. We need kjpindex words = ',kjpindex*nslm*trois
       STOP 'hydrol_init'
    END IF

    ALLOCATE (nroot(nvm, nstm, nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in nroot allocation. We stop. We need kjpindex words = ',nvm * nstm * nslm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (kfact_root(kjpindex, nslm, nstm), stat=ier)
    IF (ier .NE. 0) THEN
       WRITE (numout,*) 'error in kfact_root allocation, We stop. We need kjpindex words = ',kjpindex*nslm*nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (kfact(nslm, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in kfact allocation. We stop. We need kjpindex words = ',nslm * nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (zz(nslm+1, nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in zz allocation. We stop. We need kjpindex words = ',(nslm+1) * nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (dz(nslm+1, nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in dz allocation. We stop. We need kjpindex words = ',(nslm+1) * nstm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (mc_lin(imin:imax, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in mc_lin allocation. We stop. We need kjpindex words = ',(imax-imin) * nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (k_lin(imin:imax, nslm, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in k_lin allocation. We stop. We need kjpindex words = ',(imax-imin) * nslm * nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (d_lin(imin:imax, nslm, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in d_lin allocation. We stop. We need kjpindex words = ',(imax-imin) * nslm * nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (a_lin(imin:imax, nslm, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in a_lin allocation. We stop. We need kjpindex words = ',(imax-imin) * nslm * nscm
       STOP 'hydrol_init'
    END IF

    ALLOCATE (b_lin(imin:imax, nslm, nscm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in b_lin allocation. We stop. We need kjpindex words = ',(imax-imin) * nslm * nscm
       STOP 'hydrol_init'
    END IF

    !
    !  If we check the water balance we need two more variables
    !
    IF ( check_waterbal ) THEN

       ALLOCATE (tot_water_beg(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_water_beg allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_water_end(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_water_end allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_flux(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_flux allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

    ENDIF
    !
    !  If we use the almaoutputs we need a few more variables
    !  tdo - they could be allocated only if alma_output, but then they should also be computed only if alma_output

    ALLOCATE (tot_watveg_beg(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_watveg_beg allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (tot_watveg_end(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_watveg_end allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (tot_watsoil_beg(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_watsoil_beg allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (tot_watsoil_end(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_watsoil_end allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (delsoilmoist(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in delsoilmoist allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (delintercept(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in delintercept. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (delswe(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in delswe. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    ENDIF
    
    ALLOCATE (swi(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in swi. We stop. We need kjpindex words = ',kjpindex
       STOP 'hydrol_init'
    ENDIF
    
    ALLOCATE (snow_beg(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow_beg allocation. We stop. We need kjpindex words =',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ALLOCATE (snow_end(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow_end allocation. We stop. We need kjpindex words =',kjpindex
       STOP 'hydrol_init'
    END IF
    
    ! open restart input file done by sechiba_init
    ! and read data from restart input file for HYDROLOGIC process

    IF (ldrestart_read) THEN

       IF (long_print) WRITE (numout,*) ' we have to read a restart file for HYDROLOGIC variables'
       !
       IF (is_root_prc) CALL ioconf_setatt('UNITS', '-')
       !
       DO jst=1,nstm
          ! var_name= "mc_1" ... "mc_3"
           WRITE (var_name,"('moistc_',i1)") jst
           IF (is_root_prc) CALL ioconf_setatt('LONG_NAME',var_name)
           CALL restget_p (rest_id, var_name, nbp_glo, nslm , 1, kjit, .TRUE., mc(:,:,jst), "gather", nbp_glo, index_g)
       END DO
       !
       IF (is_root_prc) CALL ioconf_setatt('UNITS', '-')
       DO jst=1,nstm
          DO jsl=1,nslm
             ! var_name= "us_1_01" ... "us_3_11"
             WRITE (var_name,"('us_',i1,'_',i2.2)") jst,jsl
             IF (is_root_prc) CALL ioconf_setatt('LONG_NAME',var_name)
             CALL restget_p (rest_id, var_name, nbp_glo, nvm, 1, kjit, .TRUE., us(:,:,jst,jsl), "gather", nbp_glo, index_g)
          END DO
       END DO
       !
       var_name= 'free_drain_coef'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', '-')
          CALL ioconf_setatt('LONG_NAME','Coefficient for free drainage at bottom of soil')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nstm, 1, kjit, .TRUE., free_drain_coef, "gather", nbp_glo, index_g)
       !
       var_name= 'water2infilt'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', '-')
          CALL ioconf_setatt('LONG_NAME','Remaining water to be infiltrated on top of the soil')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nstm, 1, kjit, .TRUE., water2infilt, "gather", nbp_glo, index_g)
       !
       var_name= 'ae_ns'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'kg/m^2')
          CALL ioconf_setatt('LONG_NAME','Bare soil evap on each soil type')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nstm, 1, kjit, .TRUE., ae_ns, "gather", nbp_glo, index_g)
       !
       var_name= 'snow'        
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'kg/m^2')
          CALL ioconf_setatt('LONG_NAME','Snow mass')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, 1  , 1, kjit, .TRUE., snow, "gather", nbp_glo, index_g)
       !
       var_name= 'snow_age'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'd')
          CALL ioconf_setatt('LONG_NAME','Snow age')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, 1  , 1, kjit, .TRUE., snow_age, "gather", nbp_glo, index_g)
       !
       var_name= 'snow_nobio'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'kg/m^2')
          CALL ioconf_setatt('LONG_NAME','Snow on other surface types')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nnobio  , 1, kjit, .TRUE., snow_nobio, "gather", nbp_glo, index_g)
       !
       var_name= 'snow_nobio_age'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'd')
          CALL ioconf_setatt('LONG_NAME','Snow age on other surface types')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nnobio  , 1, kjit, .TRUE., snow_nobio_age, "gather", nbp_glo, index_g)
       !
       var_name= 'vegstress'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', '-')
          CALL ioconf_setatt('LONG_NAME','Vegetation growth moisture stress')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nvm, 1, kjit, .TRUE., vegstress, "gather", nbp_glo, index_g)
       !
       var_name= 'qsintveg'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', 'kg/m^2')
          CALL ioconf_setatt('LONG_NAME','Intercepted moisture')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nvm, 1, kjit, .TRUE., qsintveg, "gather", nbp_glo, index_g)
       !
       var_name= 'resdist'
       IF (is_root_prc) THEN
          CALL ioconf_setatt('UNITS', '-')
          CALL ioconf_setatt('LONG_NAME','Distribution of reservoirs')
       ENDIF
       CALL restget_p (rest_id, var_name, nbp_glo, nvm, 1, kjit, .TRUE., resdist, "gather", nbp_glo, index_g)
       !
       IF (is_root_prc) CALL ioconf_setatt('UNITS', '-')
       DO jst=1,nstm
          ! var_name= "cvs_over_veg_1" ... "cvs_over_veg_3"
          WRITE (var_name,"('cvs_over_veg_',i1)") jst
          IF (is_root_prc) CALL ioconf_setatt('LONG_NAME',var_name)
          CALL restget_p (rest_id, var_name, nbp_glo,  nvm, 1, kjit, .TRUE., cvs_over_veg(:,:,jst), "gather",  nbp_glo, index_g)
       END DO
       !
!!$       IF (is_root_prc) CALL ioconf_setatt('UNITS', '-')
!!$       DO jst=1,nstm
!!$          ! var_name= "humrelv_1" ... "humrelv_3"
!!$          WRITE (var_name,"('humrelv_',i1)") jst
!!$          IF (is_root_prc) CALL ioconf_setatt('LONG_NAME',var_name)
!!$          CALL restget_p (rest_id, var_name, nbp_glo,  nvm, 1, kjit, .TRUE., humrelv(:,:,jst), "gather",  nbp_glo, index_g)
!!$       END DO
       !
       !
       ! get restart values if none were found in the restart file
       !
       !Config Key  = HYDROL_MOISTURE_CONTENT
       !Config Desc = Soil moisture on each soil tile and levels
       !Config Def  = 0.3
       !Config Help = The initial value of mc if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (mc, val_exp, 'HYDROL_MOISTURE_CONTENT', 0.3_r_std)
       !
       !Config Key  = US_INIT
       !Config Desc = US_NVM_NSTM_NSLM
       !Config Def  = 0.0
       !Config Help = The initial value of us if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       DO jsl=1,nslm
          CALL setvar_p (us(:,:,:,jsl), val_exp, 'US_INIT', 0.0_r_std)
       ENDDO
       !
       !Config Key  = FREE_DRAIN_COEF
       !Config Desc = Coefficient for free drainage at bottom
       !Config Def  = 1.0, 1.0, 1.0
       !Config Help = The initial value of free drainage if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (free_drain_coef, val_exp, 'FREE_DRAIN_COEF', free_drain_max)
       !
       !Config Key  = WATER_TO_INFILT
       !Config Desc = Water to be infiltrated on top of the soil
       !Config Def  = 0.0
       !Config Help = The initial value of free drainage if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (water2infilt, val_exp, 'WATER_TO_INFILT', 0.0_r_std)
       !
       !Config Key  = EVAPNU_SOIL
       !Config Desc = Bare soil evap on each soil if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of bare soils evap if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (ae_ns, val_exp, 'EVAPNU_SOIL', 0.0_r_std)
       !
       !Config Key  = HYDROL_SNOW
       !Config Desc = Initial snow mass if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of snow mass if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (snow, val_exp, 'HYDROL_SNOW', 0.0_r_std)
       !
       !Config Key  = HYDROL_SNOWAGE
       !Config Desc = Initial snow age if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of snow age if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (snow_age, val_exp, 'HYDROL_SNOWAGE', 0.0_r_std)
       !
       !Config Key  = HYDROL_SNOW_NOBIO
       !Config Desc = Initial snow amount on ice, lakes, etc. if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of snow if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (snow_nobio, val_exp, 'HYDROL_SNOW_NOBIO', 0.0_r_std)
       !
       !Config Key  = HYDROL_SNOW_NOBIO_AGE
       !Config Desc = Initial snow age on ice, lakes, etc. if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of snow age if its value is not found
       !Config        in the restart file. This should only be used if the model is 
       !Config        started without a restart file.
       !
       CALL setvar_p (snow_nobio_age, val_exp, 'HYDROL_SNOW_NOBIO_AGE', 0.0_r_std)
       !
       !Config Key  = HYDROL_QSV
       !Config Desc = Initial water on canopy if not found in restart
       !Config Def  = 0.0
       !Config Help = The initial value of moisture on canopy if its value 
       !Config        is not found in the restart file. This should only be used if
       !Config        the model is started without a restart file. 
       !
       CALL setvar_p (qsintveg, val_exp, 'HYDROL_QSV', 0.0_r_std)
       !
       ! There is no need to configure the initialisation of resdist. If not available it is the vegetation map
       !
       IF ( MINVAL(resdist) .EQ.  MAXVAL(resdist) .AND. MINVAL(resdist) .EQ. val_exp) THEN
          resdist = veget
       ENDIF
       !
       !  Remember that it is only frac_nobio + SUM(veget(,:)) that is equal to 1. Thus we need vegtot
       !
       DO ji = 1, kjpindex
          vegtot(ji) = SUM(veget(ji,:))
       ENDDO
       !
       !
       ! compute the masks for veget


       mask_veget(:,:) = 0
       mask_soiltile(:,:) = 0

       DO jst=1,nstm
          DO ji= 1, kjpindex
             IF(soiltile(ji,jst) .GT. min_sechiba) THEN
                mask_soiltile(ji,jst) = 1
             ENDIF
          ENDDO
       ENDDO
       
       DO jv = 1, nvm
          DO ji = 1, kjpindex
             IF(veget(ji,jv) .GT. min_sechiba) THEN
                mask_veget(ji,jv) = 1
             ENDIF
          END DO
       END DO

       ! set humrelv from us

       humrelv(:,:,:) = SUM(us,dim=4)
!       humrelv_temp(:,:,:) = SUM(us,dim=4)
!       CALL setvar_p (humrelv, val_exp, 'NO_KEYWORD', humrelv_temp)
       vegstressv(:,:,:) = humrelv(:,:,:)
       ! set humrel from humrelv, assuming equi-repartition for the first time step

       humrel(:,:) = zero
       CALL setvar_p (cvs_over_veg, val_exp, 'NO_KEYWORD', un)

       DO jst=1,nstm
          DO jv=1,nvm
             DO ji=1,kjpindex

                vegstress(ji,jv)=vegstress(ji,jv) + vegstressv(ji,jv,jst) * soiltile(ji,jst)

                humrel(ji,jv)=humrel(ji,jv) + humrelv(ji,jv,jst) * & 
                     & soiltile(ji,jst) &
                     & * cvs_over_veg(ji,jv,jst)*vegtot(ji)
                humrel(ji,jv)=MAX(humrel(ji,jv), zero)* mask_veget(ji,jv)           
             END DO
          END DO
       END DO
    ENDIF
    !
    !
    IF (long_print) WRITE (numout,*) ' hydrol_init done '
    !
  END SUBROUTINE hydrol_init
  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  SUBROUTINE hydrol_clear()

    l_first_hydrol=.TRUE.

    ! Allocation for soiltile related parameters
    IF ( ALLOCATED (nvan)) DEALLOCATE (nvan)
    IF ( ALLOCATED (avan)) DEALLOCATE (avan)
    IF ( ALLOCATED (mcr)) DEALLOCATE (mcr)
    IF ( ALLOCATED (mcs)) DEALLOCATE (mcs)
    IF ( ALLOCATED (ks)) DEALLOCATE (ks)
    IF ( ALLOCATED (ds)) DEALLOCATE (ds)
    IF ( ALLOCATED (pcent)) DEALLOCATE (pcent)
    IF ( ALLOCATED (free_drain_max)) DEALLOCATE (free_drain_max)
    IF ( ALLOCATED (mcf)) DEALLOCATE (mcf)
    IF ( ALLOCATED (mcw)) DEALLOCATE (mcw)
    IF ( ALLOCATED (mc_awet)) DEALLOCATE (mc_awet)
    IF ( ALLOCATED (mc_adry)) DEALLOCATE (mc_adry)
    IF ( ALLOCATED (psis)) DEALLOCATE (psis)
    ! Other arrays
    IF (ALLOCATED (mask_veget)) DEALLOCATE (mask_veget)
    IF (ALLOCATED (mask_soiltile)) DEALLOCATE (mask_soiltile)
    IF (ALLOCATED (humrelv)) DEALLOCATE (humrelv)
    IF (ALLOCATED (vegstressv)) DEALLOCATE (vegstressv)
    IF (ALLOCATED (us)) DEALLOCATE (us)
    IF (ALLOCATED  (precisol)) DEALLOCATE (precisol)
    IF (ALLOCATED  (precisol_ns)) DEALLOCATE (precisol_ns)
    IF (ALLOCATED  (free_drain_coef)) DEALLOCATE (free_drain_coef)
    IF (ALLOCATED  (frac_bare_ns)) DEALLOCATE (frac_bare_ns)
    IF (ALLOCATED  (tot_frac_bare)) DEALLOCATE (tot_frac_bare)
    IF (ALLOCATED  (water2infilt)) DEALLOCATE (water2infilt)
    IF (ALLOCATED  (ae_ns)) DEALLOCATE (ae_ns)
    IF (ALLOCATED  (evap_bare_lim_ns)) DEALLOCATE (evap_bare_lim_ns)
    IF (ALLOCATED  (rootsink)) DEALLOCATE (rootsink)
    IF (ALLOCATED  (subsnowveg)) DEALLOCATE (subsnowveg)
    IF (ALLOCATED  (subsnownobio)) DEALLOCATE (subsnownobio)
    IF (ALLOCATED  (snowmelt)) DEALLOCATE (snowmelt)
    IF (ALLOCATED  (icemelt)) DEALLOCATE (icemelt)
    IF (ALLOCATED  (subsinksoil)) DEALLOCATE (subsinksoil)
    IF (ALLOCATED  (mx_eau_var)) DEALLOCATE (mx_eau_var)
    IF (ALLOCATED  (vegtot)) DEALLOCATE (vegtot)
    IF (ALLOCATED  (resdist)) DEALLOCATE (resdist)
    IF (ALLOCATED  (tot_water_beg)) DEALLOCATE (tot_water_beg)
    IF (ALLOCATED  (tot_water_end)) DEALLOCATE (tot_water_end)
    IF (ALLOCATED  (tot_flux)) DEALLOCATE (tot_flux)
    IF (ALLOCATED  (tot_watveg_beg)) DEALLOCATE (tot_watveg_beg)
    IF (ALLOCATED  (tot_watveg_end)) DEALLOCATE (tot_watveg_end)
    IF (ALLOCATED  (tot_watsoil_beg)) DEALLOCATE (tot_watsoil_beg)
    IF (ALLOCATED  (tot_watsoil_end)) DEALLOCATE (tot_watsoil_end)
    IF (ALLOCATED  (delsoilmoist)) DEALLOCATE (delsoilmoist)
    IF (ALLOCATED  (delintercept)) DEALLOCATE (delintercept)
    IF (ALLOCATED  (snow_beg)) DEALLOCATE (snow_beg)
    IF (ALLOCATED  (snow_end)) DEALLOCATE (snow_end)
    IF (ALLOCATED  (delswe)) DEALLOCATE (delswe)
    IF (ALLOCATED  (swi)) DEALLOCATE (swi)
    ! more allocation for cwrr scheme
    IF (ALLOCATED  (v1)) DEALLOCATE (v1)
!    IF (ALLOCATED  (vB)) DEALLOCATE (vB)
    IF (ALLOCATED  (humtot)) DEALLOCATE (humtot)
    IF (ALLOCATED  (flux)) DEALLOCATE (flux)
    IF (ALLOCATED  (resolv)) DEALLOCATE (resolv)
    IF (ALLOCATED  (k)) DEALLOCATE (k)
    IF (ALLOCATED  (a)) DEALLOCATE (a)
    IF (ALLOCATED  (b)) DEALLOCATE (b)
    IF (ALLOCATED  (d)) DEALLOCATE (d)
    IF (ALLOCATED  (e)) DEALLOCATE (e)
    IF (ALLOCATED  (f)) DEALLOCATE (f)
    IF (ALLOCATED  (g1)) DEALLOCATE (g1)
    IF (ALLOCATED  (ep)) DEALLOCATE (ep)
    IF (ALLOCATED  (fp)) DEALLOCATE (fp)
    IF (ALLOCATED  (gp)) DEALLOCATE (gp)
    IF (ALLOCATED  (rhs)) DEALLOCATE (rhs)
    IF (ALLOCATED  (srhs)) DEALLOCATE (srhs)
    IF (ALLOCATED  (gam)) DEALLOCATE (gam)
    IF (ALLOCATED  (tmc)) DEALLOCATE (tmc)
    IF (ALLOCATED  (tmcs)) DEALLOCATE (tmcs)
    IF (ALLOCATED  (tmcr)) DEALLOCATE (tmcr)
    IF (ALLOCATED  (tmc_litter)) DEALLOCATE (tmc_litter)
    IF (ALLOCATED  (tmc_litt_mea)) DEALLOCATE (tmc_litt_mea)
    IF (ALLOCATED  (tmc_litter_res)) DEALLOCATE (tmc_litter_res)
    IF (ALLOCATED  (tmc_litter_wilt)) DEALLOCATE (tmc_litter_wilt)
    IF (ALLOCATED  (tmc_litter_field)) DEALLOCATE (tmc_litter_field)
    IF (ALLOCATED  (tmc_litter_sat)) DEALLOCATE (tmc_litter_sat)
    IF (ALLOCATED  (tmc_litter_awet)) DEALLOCATE (tmc_litter_awet)
    IF (ALLOCATED  (tmc_litter_adry)) DEALLOCATE (tmc_litter_adry)
    IF (ALLOCATED  (tmc_litt_wet_mea)) DEALLOCATE (tmc_litt_wet_mea)
    IF (ALLOCATED  (tmc_litt_dry_mea)) DEALLOCATE (tmc_litt_dry_mea)
    IF (ALLOCATED  (qflux00)) DEALLOCATE (qflux00)
    IF (ALLOCATED  (ru_ns)) DEALLOCATE (ru_ns)
    IF (ALLOCATED  (dr_ns)) DEALLOCATE (dr_ns)
    IF (ALLOCATED  (tr_ns)) DEALLOCATE (tr_ns)
    IF (ALLOCATED  (cvs_over_veg)) DEALLOCATE (cvs_over_veg)
    IF (ALLOCATED  (corr_veg_soil)) DEALLOCATE (corr_veg_soil)
    IF (ALLOCATED  (mc)) DEALLOCATE (mc)
    IF (ALLOCATED  (soilmoist)) DEALLOCATE (soilmoist)
    IF (ALLOCATED  (soil_wet)) DEALLOCATE (soil_wet)
    IF (ALLOCATED  (soil_wet_litter)) DEALLOCATE (soil_wet_litter)
    IF (ALLOCATED  (qflux)) DEALLOCATE (qflux)
    IF (ALLOCATED  (tmat)) DEALLOCATE (tmat)
    IF (ALLOCATED  (stmat)) DEALLOCATE (stmat)
    IF (ALLOCATED  (nroot)) DEALLOCATE (nroot)
    IF (ALLOCATED  (kfact_root)) DEALLOCATE (kfact_root)
    IF (ALLOCATED  (kfact)) DEALLOCATE (kfact)
    IF (ALLOCATED  (zz)) DEALLOCATE (zz)
    IF (ALLOCATED  (dz)) DEALLOCATE (dz)
    IF (ALLOCATED  (mc_lin)) DEALLOCATE (mc_lin)
    IF (ALLOCATED  (k_lin)) DEALLOCATE (k_lin)
    IF (ALLOCATED  (d_lin)) DEALLOCATE (d_lin)
    IF (ALLOCATED  (a_lin)) DEALLOCATE (a_lin)
    IF (ALLOCATED  (b_lin)) DEALLOCATE (b_lin)

    RETURN 

  END SUBROUTINE hydrol_clear

  !! This routine initializes HYDROLOGIC variables
  !! - mx_eau_var
  SUBROUTINE hydrol_var_init (kjpindex, veget, soiltile, njsc, mx_eau_var, shumdiag, k_litt, &
       & litterhumdiag, drysoil_frac, evap_bare_lim) 

    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                          :: kjpindex      !! domain size
    ! input fields
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: veget         !! fraction of vegetation type
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: njsc            !! Indeces of the soiltype
    REAL(r_std), DIMENSION (kjpindex,nstm), INTENT (in) :: soiltile      !! Map of soil types
    ! modified fields
    ! output fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: mx_eau_var    !!
    REAL(r_std),DIMENSION (kjpindex,nbdl), INTENT (out) :: shumdiag      !! relative soil moisture
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: k_litt        !! litter cond.
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: litterhumdiag !! litter humidity
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: drysoil_frac  !! function of litter humidity
    REAL(r_std),DIMENSION (kjpindex), INTENT(out)       :: evap_bare_lim !! 
    ! local declaration

    INTEGER(i_std)                                     :: ji, jv, jd, jst, jsc, jsl, ji_soil, i
    REAL(r_std)                                         :: m, frac
    REAL(r_std)                                         :: avan_mod, nvan_mod !! modified VG parameters with exponantial profile
    REAL(r_std), DIMENSION(nslm,nscm)                   :: afact, nfact     !! multiplicative factor for decay of a and n 
    ! parameters for "soil densification" with depth
    REAL(r_std)                              :: dp_comp     !! depth at which the 'compacted value' (Van Genuchten) of ksat is reached
    REAL(r_std)                              :: f_ks        !! exponential factor for decay of ksat with depth
    ! Fixed parameters from fitted relationships
    REAL(r_std), PARAMETER                              :: n0 = 0.95        !! fitted value for relation log((n-n0)/(n_ref-n0)) = nk_rel * log(k/k_ref)
    REAL(r_std), PARAMETER                              :: nk_rel = 0.34    !! fitted value for relation log((n-n0)/(n_ref-n0)) = nk_rel * log(k/k_ref)
    REAL(r_std), PARAMETER                              :: a0 = 0.00012      !! fitted value for relation log((a-a0)/(a_ref-a0)) = ak_rel * log(k/k_ref)
    REAL(r_std), PARAMETER                              :: ak_rel = 0.53    !! fitted value for relation log((a-a0)/(a_ref-a0)) = ak_rel * log(k/k_ref)
    REAL(r_std)                              :: kfact_max       !! Maximum factor for K due to depth
    !
    !
    ! Calcul the matrix coef for dublin model:
    ! pice-wise linearised hydraulic conductivity k_lin=alin * mc_lin + b_lin
    ! and diffusivity d_lin in each interval of mc, called mc_lin,
    ! between imin, for residual mcr, 
    ! and imax for saturation mcs.
    !
    !Config Key  = MAXK_PARAM
    !Config Desc = Maximum Factor for Ks increase due to vegetation
    !Config Def  = 10.0
    !Config Help =  
    !
    kfact_max = 10.0
    CALL getin_p ('MAXK_PARAM', kfact_max)
    !
    !Config Key  = KPROF_PARAM
    !Config Desc = Factor for Ks decay with depth
    !Config Def  = 2.0
    !Config Help =  
    !
    f_ks = deux
    CALL getin_p ('KPROF_PARAM', f_ks)
    !
    !Config Key  = KDEPTH_PARAM
    !Config Desc = Depth for compacted value of Ks 
    !Config Def  = 0.3
    !Config Help =  
    !
    dp_comp = 0.3
    CALL getin_p ('KDEPTH_PARAM', dp_comp)
    !
    !
    DO jst=1,nstm
       !-
       !- 1. compute the depths
       !-
       zz(1,jst) = zero
       dz(1,jst) = zero
       DO jsl=2,nslm
          zz(jsl,jst) = dpu_max* mille*((2**(jsl-1))-1)/ ((2**(nslm-1))-1)
          dz(jsl,jst) = zz(jsl,jst)-zz(jsl-1,jst)
       ENDDO
       zz(nslm+1,jst) = zz(nslm,jst)
       dz(nslm+1,jst) = zero
       !-
       !- 3. compute the profile for roots
       !-
       DO jv = 1,nvm
          DO jsl = 2, nslm-1
             nroot(jv,jst,jsl) = (EXP(-humcste(jv)*zz(jsl,jst)/mille)) * &
                     & (EXP(humcste(jv)*dz(jsl,jst)/mille/deux) - &
                     & EXP(-humcste(jv)*dz(jsl+1,jst)/mille/deux))/ &
                     & (EXP(-humcste(jv)*dz(2,jst)/mille/deux) &
                     & -EXP(-humcste(jv)*zz(nslm,jst)/mille))
          ENDDO
       ENDDO
       DO jv=1,nvm
          nroot(jv,jst,1) = zero
          nroot(jv,jst,nslm) = (EXP(humcste(jv)*dz(nslm,jst)/mille/deux) -un) * &
                  & EXP(-humcste(jv)*zz(nslm,jst)/mille) / &
                  & (EXP(-humcste(jv)*dz(2,jst)/mille/deux) &
                  & -EXP(-humcste(jv)*zz(nslm,jst)/mille))
       ENDDO
       ! An additional exponential factor for ks depending on the amount of roots in the soil 
       ! through a geometric average over the vegets
       kfact_root(:,:,jst) = un
       DO jsl = 1, nslm
          DO jv = 2, nvm
             DO ji = 1, kjpindex
                kfact_root(ji,jsl,jst) = kfact_root(ji,jsl,jst) * &
                     & MAX((MAXVAL(ks_usda)/ks(njsc(ji)))**(- veget(ji,jv) * (humcste(jv)*zz(jsl,jst)/mille - un)/deux), un) 
             ENDDO
          ENDDO
       ENDDO
    ENDDO

    !-
    !- 4. Compute the profile for ksat, a and n
    !-
    ! This is used for FAO2 map (with soil type 2 and 4 being heterogeneous with depth)
    IF (nscm .EQ. 5) THEN
       ! For every soil texture
       DO jsc = 1, nscm
          DO jsl=1,nslm
             frac = MAX(zero, MIN(un, f_ks * (zz(jsl,jsc)/mille - dp_comp)))
             kfact(jsl,jsc) = ( ks(MIN(nscm,jsc+1))/ks(jsc) )**frac
             nfact(jsl,jsc) = ( (nvan(MIN(nscm,jsc+1))-n0)/(nvan(jsc)-n0) )**frac
             afact(jsl,jsc) = ( (avan(MIN(nscm,jsc+1))-a0)/(avan(jsc)-a0) )**frac
          ENDDO
       ENDDO
    ! And this is to put a global soiltype profile 
    ELSE
       ! For every soil texture
       DO jsc = 1, nscm
          DO jsl=1,nslm
             kfact(jsl,jsc) = MIN(MAX(EXP(- f_ks * (zz(jsl,jsc)/mille - dp_comp)), un/kfact_max),un)
             nfact(jsl,jsc) = ( kfact(jsl,jsc) )**nk_rel
             afact(jsl,jsc) = ( kfact(jsl,jsc) )**ak_rel
          ENDDO
       ENDDO
    ENDIF
    
    ! For every soil texture
    DO jsc = 1, nscm
       !-
       !- 5. compute the linearized values of k, a, b and d
       !-
       mc_lin(imin,jsc)=mcr(jsc)
       mc_lin(imax,jsc)=mcs(jsc)
       DO ji= imin+1, imax-1 
          mc_lin(ji,jsc) = mcr(jsc) + (ji-imin)*(mcs(jsc)-mcr(jsc))/(imax-imin)
       ENDDO

       DO jsl = 1, nslm
          nvan_mod = n0 + (nvan(jsc)-n0) * nfact(jsl,jsc)
          avan_mod = a0 + (avan(jsc)-a0) * afact(jsl,jsc)
          m = un - un / nvan_mod
          ! Perhaps we may have problems with precision here for some machines (k being very small for ji=imin+1
          ! How can we handle it?
          DO ji = imax,imin+1,-1 
             frac=MIN(un,(mc_lin(ji,jsc)-mcr(jsc))/(mcs(jsc)-mcr(jsc)))
             k_lin(ji,jsl,jsc) = ks(jsc) * kfact(jsl,jsc) * (frac**0.5) * ( un - ( un - frac ** (un/m)) ** m )**2
          ENDDO
          ! We have to avoid k=0
          k_lin(imin,jsl,jsc) = k_lin(imin+1,jsl,jsc)/mille
          
          DO ji = imin, imax-1
             a_lin(ji,jsl,jsc) = (k_lin(ji+1,jsl,jsc)-k_lin(ji,jsl,jsc)) / (mc_lin(ji+1,jsc)-mc_lin(ji,jsc))
             b_lin(ji,jsl,jsc)  = k_lin(ji,jsl,jsc) - a_lin(ji,jsl,jsc)*mc_lin(ji,jsc)
             
             IF(ji.NE.imin.AND.ji.NE.imax-1) THEN
                frac=MIN(un,(mc_lin(ji,jsc)-mcr(jsc))/(mcs(jsc)-mcr(jsc)))
                d_lin(ji,jsl,jsc) =(k_lin(ji,jsl,jsc) / (avan_mod*m*nvan_mod)) *  &
                     &  ( (frac**(-un/m))/(mc_lin(ji,jsc)-mcr(jsc)) ) * &
                     &  (  frac**(-un/m) -un ) ** (-m)
                frac=MIN(un,(mc_lin(ji+1,jsc)-mcr(jsc))/(mcs(jsc)-mcr(jsc)))
                d_lin(ji+1,jsl,jsc) =(k_lin(ji+1,jsl,jsc) / (avan_mod*m*nvan_mod))*&
                     &  ( (frac**(-un/m))/(mc_lin(ji+1,jsc)-mcr(jsc)) ) * &
                     &  (  frac**(-un/m) -un ) ** (-m)
                d_lin(ji,jsl,jsc) = undemi * (d_lin(ji,jsl,jsc)+d_lin(ji+1,jsl,jsc))
             ELSEIF(ji.EQ.imin) THEN
                d_lin(ji,jsl,jsc) = zero
             ELSEIF(ji.EQ.imax-1) THEN
                frac=MIN(un,(mc_lin(ji,jsc)-mcr(jsc))/(mcs(jsc)-mcr(jsc)))
                d_lin(ji,jsl,jsc) =(k_lin(ji,jsl,jsc) / (avan_mod*m*nvan_mod)) * &
                     & ( (frac**(-un/m))/(mc_lin(ji,jsc)-mcr(jsc)) ) *  &
                     & (  frac**(-un/m) -un ) ** (-m)
                ds(jsl,jsc) = d_lin(ji,jsl,jsc)
             ENDIF
          ENDDO
       ENDDO
    ENDDO

    ! initialisation
    mx_eau_var(:) = zero
    !
    DO jst = 1,nstm
       DO ji = 1, kjpindex
          mx_eau_var(ji) = mx_eau_var(ji) + soiltile(ji,jst)*&
               &   dpu_max*mille*mcs(njsc(ji))
       END DO
    END DO

    DO ji = 1,kjpindex 
       IF (vegtot(ji) .LE. zero) THEN
          mx_eau_var(ji) = mx_eau_eau*deux
       ENDIF

    END DO

    ! Compute the litter humidity, shumdiag and fry
 
    litterhumdiag(:) = zero
    k_litt(:) = zero
    tmc_litt_mea(:) = zero
    tmc_litt_wet_mea(:) = zero
    tmc_litt_dry_mea(:) = zero
    shumdiag(:,:) = zero
    soilmoist(:,:) = zero
    humtot(:) = zero
    tmc(:,:) = zero
    swi(:) = zero

    ! Loop on soil types to compute the variables (ji,jst)

    DO jst=1,nstm 

       DO ji = 1, kjpindex
          tmcs(ji,jst)=dpu_max* mille*mcs(njsc(ji))
          tmcr(ji,jst)=dpu_max* mille*mcr(njsc(ji))
       ENDDO
    ENDDO

       ! The total soil moisture for each soil type:

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          tmc(ji,jst)= dz(2,jst) * ( trois*mc(ji,1,jst)+ mc(ji,2,jst))/huit
       ENDDO
    ENDDO

    DO jst=1,nstm 
       DO jsl=2,nslm-1
          DO ji = 1, kjpindex
             tmc(ji,jst) = tmc(ji,jst) + dz(jsl,jst) * ( trois*mc(ji,jsl,jst) + mc(ji,jsl-1,jst))/huit &
                  & + dz(jsl+1,jst)*(trois*mc(ji,jsl,jst) + mc(ji,jsl+1,jst))/huit
          ENDDO
       END DO
    ENDDO

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          tmc(ji,jst) = tmc(ji,jst) +  dz(nslm,jst) * (trois * mc(ji,nslm,jst) + mc(ji,nslm-1,jst))/huit
          tmc(ji,jst) = tmc(ji,jst) + water2infilt(ji,jst)
       ENDDO
    ENDDO

       ! The litter variables:

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          tmc_litter(ji,jst) = dz(2,jst) * (trois*mc(ji,1,jst)+mc(ji,2,jst))/huit
          tmc_litter_wilt(ji,jst) = dz(2,jst) * mcw(njsc(ji)) / deux
          tmc_litter_res(ji,jst) = dz(2,jst) * mcr(njsc(ji)) / deux
          tmc_litter_field(ji,jst) = dz(2,jst) * mcf(njsc(ji)) / deux
          tmc_litter_sat(ji,jst) = dz(2,jst) * mcs(njsc(ji)) / deux
          tmc_litter_awet(ji,jst) = dz(2,jst) * mc_awet(njsc(ji)) / deux
          tmc_litter_adry(ji,jst) = dz(2,jst) * mc_adry(njsc(ji)) / deux
       ENDDO
    ENDDO

          ! sum from level 1 to 4
          
    DO jst=1,nstm 
       DO jsl=2,4
          DO ji = 1, kjpindex
             tmc_litter(ji,jst) = tmc_litter(ji,jst) + dz(jsl,jst) * & 
                  & ( trois*mc(ji,jsl,jst) + mc(ji,jsl-1,jst))/huit &
                  & + dz(jsl+1,jst)*(trois*mc(ji,jsl,jst) + mc(ji,jsl+1,jst))/huit

             tmc_litter_wilt(ji,jst) = tmc_litter_wilt(ji,jst) + &
                  &(dz(jsl,jst)+ dz(jsl+1,jst))*& 
                  & mcw(njsc(ji))/deux
             tmc_litter_res(ji,jst) = tmc_litter_res(ji,jst) + &
                  &(dz(jsl,jst)+ dz(jsl+1,jst))*& 
                  & mcr(njsc(ji))/deux
             tmc_litter_sat(ji,jst) = tmc_litter_sat(ji,jst) + &
                  &(dz(jsl,jst)+ dz(jsl+1,jst))* & 
                  & mcs(njsc(ji))/deux
             tmc_litter_field(ji,jst) = tmc_litter_field(ji,jst) + &
                  & (dz(jsl,jst)+ dz(jsl+1,jst))* & 
                  & mcf(njsc(ji))/deux
             tmc_litter_awet(ji,jst) = tmc_litter_awet(ji,jst) + &
                  &(dz(jsl,jst)+ dz(jsl+1,jst))* & 
                  & mc_awet(njsc(ji))/deux
             tmc_litter_adry(ji,jst) = tmc_litter_adry(ji,jst) + &
                  & (dz(jsl,jst)+ dz(jsl+1,jst))* & 
                  & mc_adry(njsc(ji))/deux
          END DO
       ENDDO
    ENDDO
       ! subsequent calcul of soil_wet_litter (tmc-tmcw)/(tmcf-tmcw)

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          soil_wet_litter(ji,jst)=MIN(un, MAX(zero,&
               &(tmc_litter(ji,jst)-tmc_litter_wilt(ji,jst))/&
               & (tmc_litter_field(ji,jst)-tmc_litter_wilt(ji,jst)) ))
       ENDDO
    ENDDO

       ! subsequent calcul of soil_wet_litter (tmc-tmcw)/(tmcf-tmcw)

       ! Soil wetness profiles (mc-mcw)/(mcs-mcw)

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          soil_wet(ji,1,jst) = MIN(un, MAX(zero,&
               &(trois*mc(ji,1,jst) + mc(ji,2,jst) - quatre*mcw(njsc(ji)))&
               & /(quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
          humrelv(ji,1,jst) = zero
       ENDDO
    ENDDO

    DO jst=1,nstm 
       DO jsl=2,nslm-1
          DO ji = 1, kjpindex
             soil_wet(ji,jsl,jst) = MIN(un, MAX(zero,&
                  & (trois*mc(ji,jsl,jst) + & 
                  & mc(ji,jsl-1,jst) *(dz(jsl,jst)/(dz(jsl,jst)+dz(jsl+1,jst))) &
                  & + mc(ji,jsl+1,jst)*(dz(jsl+1,jst)/(dz(jsl,jst)+dz(jsl+1,jst))) &
                  & - quatre*mcw(njsc(ji))) / (quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
          END DO
       ENDDO
    ENDDO

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          soil_wet(ji,nslm,jst) = MIN(un, MAX(zero,&
               & (trois*mc(ji,nslm,jst) &
               & + mc(ji,nslm-1,jst)-quatre*mcw(njsc(ji)))/(quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
       ENDDO
    ENDDO

       !Now we compute the grid averaged values

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          i= MAX(MIN(INT((imax-imin)*(tmc_litter(ji,jst)-tmc_litter_res(ji,jst))&
               &         / (tmc_litter_sat(ji,jst)-tmc_litter_res(ji,jst)))+imin , imax-1), imin)
          ! k_litt is an averaged conductivity for saturated infiltration in the 'litter' layer
          ! This is used for reinfiltration from surface water
          k_litt(ji) = k_litt(ji) + soiltile(ji,jst) * SQRT(k_lin(i,1,njsc(ji))*ks(njsc(ji)))
       ENDDO
    ENDDO

    DO jst=1,nstm 
       DO ji = 1, kjpindex
          humtot(ji) = humtot(ji) + soiltile(ji,jst) * tmc(ji,jst) 

          litterhumdiag(ji) = litterhumdiag(ji) + &
               & soil_wet_litter(ji,jst) * soiltile(ji,jst)

          tmc_litt_wet_mea(ji) =  tmc_litt_wet_mea(ji) + & 
               & tmc_litter_awet(ji,jst)* soiltile(ji,jst)

          tmc_litt_dry_mea(ji) = tmc_litt_dry_mea(ji) + &
               & tmc_litter_adry(ji,jst) * soiltile(ji,jst) 

          tmc_litt_mea(ji) = tmc_litt_mea(ji) + &
               & tmc_litter(ji,jst) * soiltile(ji,jst) 
       ENDDO
    ENDDO

    DO jst=1,nstm 
       DO jsl=1,nbdl
          DO ji = 1, kjpindex
             shumdiag(ji,jsl)= shumdiag(ji,jsl) + soil_wet(ji,jsl,jst) * &
                  & ((mcs(njsc(ji))-mcw(njsc(ji)))/(mcf(njsc(ji))-mcw(njsc(ji)))) * &
                  & soiltile(ji,jst)
             soilmoist(ji,jsl) = soilmoist(ji,jsl) + mc(ji,jsl,jst)*soiltile(ji,jst)
             shumdiag(ji,jsl) = MAX(MIN(shumdiag(ji,jsl), un), zero) 
          END DO
       END DO 

    END DO ! loop on soiltile
    !
    !
    !
    DO ji=1,kjpindex
       drysoil_frac(ji) = un + MAX( MIN( (tmc_litt_dry_mea(ji) - tmc_litt_mea(ji)) / &
            & (tmc_litt_wet_mea(ji) - tmc_litt_dry_mea(ji)), zero), - un)
    END DO

    evap_bare_lim = zero

    IF (long_print) WRITE (numout,*) ' hydrol_var_init done '

  END SUBROUTINE hydrol_var_init

  !! This routine computes snow processes
  !!
  SUBROUTINE hydrol_snow (kjpindex, dtradia, precip_rain, precip_snow , temp_sol_new, soilcap,&
       & frac_nobio, totfrac_nobio, vevapnu, vevapsno, snow, snow_age, snow_nobio, snow_nobio_age, &
       & tot_melt, snowdepth)

    ! 
    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size
    REAL(r_std), INTENT (in)                                 :: dtradia       !! Time step in seconds
    ! input fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_rain   !! Rainfall
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_snow   !! Snow precipitation
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: temp_sol_new  !! New soil temperature
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: soilcap       !! Soil capacity
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(in)     :: frac_nobio    !! Fraction of continental ice, lakes, ...
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: totfrac_nobio !! Total fraction of continental ice+lakes+ ...
    ! modified fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapnu       !! Bare soil evaporation
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapsno      !! Snow evaporation
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: snow          !! Snow mass [Kg/m^2]
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: snow_age      !! Snow age
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio    !! Ice water balance
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio_age!! Snow age on ice, lakes, ...
    ! output fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: tot_melt      !! Total melt  
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: snowdepth     !! Snow depth
    !
    ! local declaration
    !
    INTEGER(i_std)                               :: ji, jv
    REAL(r_std), DIMENSION (kjpindex)             :: d_age  !! Snow age change
    REAL(r_std), DIMENSION (kjpindex)             :: xx     !! temporary
    REAL(r_std)                                   :: snowmelt_tmp !! The name says it all !

    !
    ! for continental points
    !

    !
    ! 0. initialisation
    !
    DO jv = 1, nnobio
       DO ji=1,kjpindex
          subsnownobio(ji,jv) = zero
       ENDDO
    ENDDO
    DO ji=1,kjpindex
       subsnowveg(ji) = zero
       snowmelt(ji) = zero
       icemelt(ji) = zero
       subsinksoil(ji) = zero
       tot_melt(ji) = zero
    ENDDO
    !
    ! 1. On vegetation
    !
    DO ji=1,kjpindex
       !
       ! 1.1. It is snowing
       !
       snow(ji) = snow(ji) + (un - totfrac_nobio(ji))*precip_snow(ji)
       !
       !
       ! 1.2. Sublimation - separate between vegetated and no-veget fractions 
       !      Care has to be taken as we might have sublimation from the
       !      the frac_nobio while there is no snow on the rest of the grid.
       !
       IF ( snow(ji) > snowcri ) THEN
          subsnownobio(ji,iice) = frac_nobio(ji,iice)*vevapsno(ji)
          subsnowveg(ji) = vevapsno(ji) - subsnownobio(ji,iice)
       ELSE
          ! Correction Nathalie - Juillet 2006.
          ! On doit d'abord tester s'il existe un frac_nobio!
          ! Pour le moment je ne regarde que le iice
          IF ( frac_nobio(ji,iice) .GT. min_sechiba) THEN
             subsnownobio(ji,iice) = vevapsno(ji)
             subsnowveg(ji) = zero
          ELSE 
             subsnownobio(ji,iice) = zero
             subsnowveg(ji) = vevapsno(ji)
          ENDIF
       ENDIF
       !
       !
       ! 1.2.1 Check that sublimation on the vegetated fraction is possible.
       !
       IF (subsnowveg(ji) .GT. snow(ji)) THEN
          ! What could not be sublimated goes into soil evaporation
          !         vevapnu(ji) = vevapnu(ji) + (subsnowveg(ji) - snow(ji))
          IF( (un - totfrac_nobio(ji)).GT.min_sechiba) THEN
             subsinksoil (ji) = (subsnowveg(ji) - snow(ji))/ (un - totfrac_nobio(ji))
          END IF
          ! Sublimation is thus limited to what is available
          subsnowveg(ji) = snow(ji)
          snow(ji) = zero
          vevapsno(ji) = subsnowveg(ji) + subsnownobio(ji,iice)
       ELSE
          snow(ji) = snow(ji) - subsnowveg(ji)
       ENDIF
       !
       ! 1.3. snow melt only if temperature positive
       !
       IF (temp_sol_new(ji).GT.tp_00) THEN
          !
          IF (snow(ji).GT.sneige) THEN
             !
             snowmelt(ji) = (1. - frac_nobio(ji,iice))*(temp_sol_new(ji) - tp_00) * soilcap(ji) / chalfu0
             !
             ! 1.3.1.1 enough snow for melting or not
             !
             IF (snowmelt(ji).LT.snow(ji)) THEN
                snow(ji) = snow(ji) - snowmelt(ji)
             ELSE
                snowmelt(ji) = snow(ji)
                snow(ji) = zero
             END IF
             !
          ELSEIF (snow(ji).GE.zero) THEN
             !
             ! 1.3.2 not enough snow
             !
             snowmelt(ji) = snow(ji)
             snow(ji) = zero
          ELSE
             !
             ! 1.3.3 negative snow - now snow melt
             !
             snow(ji) = zero
             snowmelt(ji) = zero
             WRITE(numout,*) 'hydrol_snow: WARNING! snow was negative and was reset to zero. '
             !
          END IF

       ENDIF
       !
       ! 1.4. Ice melt only if there is more than a given mass : maxmass_glacier,
       !      i.e. only weight melts glaciers !
       ! Ajouts Edouard Davin / Nathalie de Noblet add extra to melting
       !
       IF ( snow(ji) .GT. maxmass_glacier ) THEN
          snowmelt(ji) = snowmelt(ji) + (snow(ji) - maxmass_glacier)
          snow(ji) = maxmass_glacier
       ENDIF
       !
    END DO
    !
    ! 2. On Land ice
    !
    DO ji=1,kjpindex
       !
       ! 2.1. It is snowing
       !
       snow_nobio(ji,iice) = snow_nobio(ji,iice) + frac_nobio(ji,iice)*precip_snow(ji) + &
            & frac_nobio(ji,iice)*precip_rain(ji)
       !
       ! 2.2. Sublimation - was calculated before it can give us negative snow_nobio but that is OK
       !      Once it goes below a certain values (-maxmass_glacier for instance) we should kill
       !      the frac_nobio(ji,iice) !
       !
       snow_nobio(ji,iice) = snow_nobio(ji,iice) - subsnownobio(ji,iice)
       !
       ! 2.3. Snow melt only for continental ice fraction
       !
       snowmelt_tmp = zero
       IF (temp_sol_new(ji) .GT. tp_00) THEN
          !
          ! 2.3.1 If there is snow on the ice-fraction it can melt
          !
          snowmelt_tmp = frac_nobio(ji,iice)*(temp_sol_new(ji) - tp_00) * soilcap(ji) / chalfu0
          !
          IF ( snowmelt_tmp .GT. snow_nobio(ji,iice) ) THEN
             snowmelt_tmp = MAX( zero, snow_nobio(ji,iice))
          ENDIF
          snowmelt(ji) = snowmelt(ji) + snowmelt_tmp
          snow_nobio(ji,iice) = snow_nobio(ji,iice) - snowmelt_tmp
          !
       ENDIF
       !
       ! 2.4 Ice melt only if there is more than a given mass : maxmass_glacier, 
       !     i.e. only weight melts glaciers !
       !
       IF ( snow_nobio(ji,iice) .GT. maxmass_glacier ) THEN
          icemelt(ji) = snow_nobio(ji,iice) - maxmass_glacier
          snow_nobio(ji,iice) = maxmass_glacier
       ENDIF
       !
    END DO

    !
    ! 3. On other surface types - not done yet
    !
    IF ( nnobio .GT. 1 ) THEN
       WRITE(numout,*) 'WE HAVE',nnobio-1,' SURFACE TYPES I DO NOT KNOW'
       WRITE(numout,*) 'CANNOT TREAT SNOW ON THESE SURFACE TYPES'
       STOP 'in hydrol_snow' 
    ENDIF

    !
    ! 4. computes total melt (snow and ice)
    !
    DO ji = 1, kjpindex
       tot_melt(ji) = icemelt(ji) + snowmelt(ji)
    ENDDO

    !
    ! 5. computes snow age on veg and ice (for albedo)
    !
    DO ji = 1, kjpindex
       !
       ! 5.1 Snow age on vegetation
       !
       IF (snow(ji) .LE. zero) THEN
          snow_age(ji) = zero
       ELSE
          snow_age(ji) =(snow_age(ji) + (un - snow_age(ji)/max_snow_age) * dtradia/one_day) &
               & * EXP(-precip_snow(ji) / snow_trans)
       ENDIF
       !
       ! 5.2 Snow age on ice
       !
       ! age of snow on ice: a little bit different because in cold regions, we really
       ! cannot negect the effect of cold temperatures on snow metamorphism any more.
       !
       IF (snow_nobio(ji,iice) .LE. zero) THEN
          snow_nobio_age(ji,iice) = zero
       ELSE
          !
          d_age(ji) = ( snow_nobio_age(ji,iice) + &
               &  (un - snow_nobio_age(ji,iice)/max_snow_age) * dtradia/one_day ) * &
               &  EXP(-precip_snow(ji) / snow_trans) - snow_nobio_age(ji,iice)
          IF (d_age(ji) .GT. min_sechiba ) THEN
             xx(ji) = MAX( tp_00 - temp_sol_new(ji), zero )
             xx(ji) = ( xx(ji) / 7._r_std ) ** 4._r_std
             d_age(ji) = d_age(ji) / (un+xx(ji))
          ENDIF
          snow_nobio_age(ji,iice) = MAX( snow_nobio_age(ji,iice) + d_age(ji), zero )
          !
       ENDIF

    ENDDO

    !
    ! 6.0 Diagnose the depth of the snow layer
    !

    DO ji = 1, kjpindex
       snowdepth(ji) = snow(ji) /sn_dens
    ENDDO

    IF (long_print) WRITE (numout,*) ' hydrol_snow done '

  END SUBROUTINE hydrol_snow

  !! This routine computes canopy processes
  !!
  SUBROUTINE hydrol_canop (kjpindex, precip_rain, vevapwet, veget, qsintmax, &
       & qsintveg,precisol,tot_melt)

    ! 
    ! interface description
    !
    INTEGER(i_std), INTENT(in)                               :: kjpindex    !! Domain size
    ! input fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_rain !! Rain precipitation
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: vevapwet    !! Interception loss
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: veget       !! Fraction of vegetation type 
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: qsintmax    !! Maximum water on vegetation for interception
    REAL(r_std), DIMENSION  (kjpindex), INTENT (in)          :: tot_melt         !! Total melt
    ! modified fields
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)     :: qsintveg    !! Water on vegetation due to interception
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(out)           :: precisol    !! Eau tombee sur le sol
    ! output fields

    !
    ! local declaration
    !
    INTEGER(i_std)                                :: ji, jv
    REAL(r_std), DIMENSION (kjpindex,nvm)          :: zqsintvegnew
    LOGICAL, SAVE                                  :: firstcall=.TRUE.
    REAL(r_std), SAVE, DIMENSION(nvm)              :: throughfall_by_pft

    IF ( firstcall ) THEN
       !Config  Key  = PERCENT_THROUGHFALL_PFT
       !Config  Desc = Percent by PFT of precip that is not intercepted by the canopy
       !Config  Def  = 30. 30. 30. 30. 30. 30. 30. 30. 30. 30. 30. 30. 30.
       !Config  Help = During one rainfall event, PERCENT_THROUGHFALL_PFT% of the incident rainfall
       !Config         will get directly to the ground without being intercepted, for each PFT.
       
       throughfall_by_pft = (/ 30., 30., 30., 30., 30., 30., 30., 30., 30., 30., 30., 30., 30. /)
       CALL getin_p('PERCENT_THROUGHFALL_PFT',throughfall_by_pft)
       throughfall_by_pft = throughfall_by_pft / 100.

       firstcall=.FALSE.
    ENDIF


    ! calcul de qsintmax a prevoir a chaque pas de temps
    ! dans ini_sechiba
    ! boucle sur les points continentaux
    ! calcul de qsintveg au pas de temps suivant
    ! par ajout du flux interception loss
    ! calcule par enerbil en fonction
    ! des calculs faits dans diffuco
    ! calcul de ce qui tombe sur le sol
    ! avec accumulation dans precisol
    ! essayer d'harmoniser le traitement du sol nu
    ! avec celui des differents types de vegetation
    ! fait si on impose qsintmax ( ,1) = 0.0
    !
    ! loop for continental subdomain
    !
    !
    ! 1. evaporation off the continents
    !
    ! 1.1 The interception loss is take off the canopy. 
    DO jv=1,nvm
       qsintveg(:,jv) = qsintveg(:,jv) - vevapwet(:,jv)
    END DO

    ! 1.2 It is raining : precip_rain is shared for each vegetation
    ! type
    !     sum (veget (1,nvm)) must be egal to 1-totfrac_nobio.
    !     iniveget computes veget each day
    !
    DO jv=1,nvm
       ! Correction Nathalie - Juin 2006 - une partie de la pluie arrivera toujours sur le sol
       ! sorte de throughfall supplementaire
       !Ancienne formulation
       qsintveg(:,jv) = qsintveg(:,jv) + veget(:,jv) * precip_rain(:)
       !Nouvelle formulation
       !qsintveg(:,jv) = qsintveg(:,jv) + veget(:,jv) * ((1-throughfall_by_pft(jv))*precip_rain(:))
    END DO

    !
    ! 1.3 Limits the effect and sum what receives soil
    !
    precisol(:,:) = zero
    DO jv=1,nvm
       DO ji = 1, kjpindex
          zqsintvegnew(ji,jv) = MIN (qsintveg(ji,jv),qsintmax(ji,jv)) 
          ! correction throughfall Nathalie - Juin 2006
          !Ancienne formulation          
          precisol(ji,jv) = qsintveg(ji,jv ) - zqsintvegnew (ji,jv)
          !Nouvelle formulation
          !precisol(ji,jv) = (veget(ji,jv)*throughfall_by_pft(jv)*precip_rain(ji)) + qsintveg(ji,jv ) - zqsintvegnew (ji,jv)
       ENDDO
    END DO
    !    
    DO jv=1,nvm
       DO ji = 1, kjpindex
          IF (vegtot(ji).GT.min_sechiba) THEN
             precisol(ji,jv) = precisol(ji,jv)+tot_melt(ji)*veget(ji,jv)/vegtot(ji)
          ENDIF
       ENDDO
    END DO
    !   
    !
    ! 1.4 swap qsintveg to the new value
    !

    DO jv=1,nvm
       qsintveg(:,jv) = zqsintvegnew (:,jv)
    END DO

    IF (long_print) WRITE (numout,*) ' hydrol_canop done '

  END SUBROUTINE hydrol_canop
  !!
  !!
  !!
  SUBROUTINE hydrol_vegupd(kjpindex, veget, frac_bare, soiltile,qsintveg,resdist)
    !
    !   The vegetation cover has changed and we need to adapt the reservoir distribution 
    !   and the distribution of plants on different soil types.
    !   You may note that this occurs after evaporation and so on have been computed. It is
    !   not a problem as a new vegetation fraction will start with humrel=0 and thus will have no
    !   evaporation. If this is not the case it should have been caught above.
    !
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex 
    ! input fields
    REAL(r_std), DIMENSION (kjpindex, nvm), INTENT(in)       :: veget            !! New vegetation map
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)        :: frac_bare        !! fraction of bare soil per veg. 
    REAL(r_std), DIMENSION (kjpindex,nstm), INTENT (in)      :: soiltile         !! Map of soil types : proportion of each soil type
    ! modified fields
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (inout)     :: qsintveg           !! Water on vegetation
    REAL(r_std), DIMENSION (kjpindex, nvm), INTENT(inout)    :: resdist          !! Old vegetation map
    !
    ! local declaration
    !
    INTEGER(i_std)                                :: ji,jv,jst,jst_pref
    REAL(r_std), DIMENSION (kjpindex,nstm)         :: soil_exist,soil_exist_max
    REAL(r_std), DIMENSION (kjpindex,nvm)          :: veget_exist,veget_exist_max
    REAL(r_std), DIMENSION (kjpindex,nvm)          :: qsintveg2                   !! Water on vegetation due to interception over old veget
    REAL(r_std), DIMENSION (kjpindex,nvm)          :: vmr                         !! variation of veget
    REAL(r_std), DIMENSION (kjpindex,nvm)          :: qsdq
    REAL(r_std), DIMENSION (kjpindex)              :: vegchtot,vtr, qstr, fra
    REAL(r_std), DIMENSION (kjpindex)              :: test_pref,denom
    REAL(r_std), PARAMETER                         :: EPS1 = EPSILON(un)
    !
    DO jv = 1, nvm
       DO ji = 1, kjpindex
!mask
!           vmr(ji,jv) = MAX ( EPSILON(un), MIN (  veget(ji,jv)-resdist(ji,jv) , MAX( EPSILON(un), veget(ji,jv)-resdist(ji,jv))  ) )
                
!            vmr(ji,jv) = MAX ( EPSILON(un),  MAX( EPSILON(un), veget(ji,jv)-resdist(ji,jv))  )
!            IF(ABS(veget(ji,jv)-resdist(ji,jv)).gt.epsilon(un)) then
!    WRITE(numout,*) '-----------------------------------------------'
!    WRITE(numout,*) 'vmr,epsilon(un),veget,resdist',vmr(ji,jv),epsilon(un)
!    WRITE(numout,*),veget(ji,jv),resdist(ji,jv)
!    WRITE(numout,*) 'ABS(veget -resdist',ABS(veget(ji,jv)-resdist(ji,jv))
!      endif
          IF ( ABS(veget(ji,jv)-resdist(ji,jv)) .GT. EPS1 ) THEN
             vmr(ji,jv) = veget(ji,jv)-resdist(ji,jv)
          ELSE
             vmr(ji,jv) = zero
          ENDIF
          !
          IF (resdist(ji,jv) .GT. min_sechiba) THEN
             qsintveg2(ji,jv) = qsintveg(ji,jv)/resdist(ji,jv)
          ELSE
             qsintveg2(ji,jv) = zero
          ENDIF
       ENDDO
    ENDDO
    !
    vegchtot(:) = zero
    DO jv = 1, nvm
       DO ji = 1, kjpindex
          vegchtot(ji) = vegchtot(ji) + ABS( vmr(ji,jv) )
       ENDDO
    ENDDO
    !
    DO jv = 1, nvm
       DO ji = 1, kjpindex
          IF ( vegchtot(ji) .GT. min_sechiba ) THEN
             qsdq(ji,jv) = ABS(vmr(ji,jv)) * qsintveg2(ji,jv)
          ENDIF
       ENDDO
    ENDDO
    !
    ! calculate water mass that we have to redistribute
    !
    qstr(:) = zero
    vtr(:) = zero
    !
    !
    DO jv = 1, nvm
       DO ji = 1, kjpindex
          IF ( ( vegchtot(ji) .GT. min_sechiba ) .AND. ( vmr(ji,jv) .LT. -min_sechiba ) ) THEN
             qstr(ji) = qstr(ji) + qsdq(ji,jv)
             vtr(ji) = vtr(ji) - vmr(ji,jv)
          ENDIF
       ENDDO
    ENDDO
    !
    ! put it into reservoir of plant whose surface area has grown
    DO jv = 1, nvm
       DO ji = 1, kjpindex
          IF ( vegchtot(ji) .GT. min_sechiba .AND. ABS(vtr(ji)) .GT. EPSILON(un)) THEN
             fra(ji) = vmr(ji,jv) / vtr(ji)
             IF ( vmr(ji,jv) .GT. min_sechiba)  THEN
                qsintveg(ji,jv) = qsintveg(ji,jv) + fra(ji)* qstr(ji)
             ELSE
                qsintveg(ji,jv) = qsintveg(ji,jv) - qsdq(ji,jv)
             ENDIF
          ENDIF
       ENDDO
    ENDDO
!MM underflow :
    DO jv = 1, nvm
      DO ji = 1, kjpindex
         IF ( ABS(qsintveg(ji,jv)) < EPS1 ) THEN
            qsintveg(ji,jv) = EPS1
         ENDIF
      ENDDO
   ENDDO

    !   Now that the work is done resdist needs an update !
   resdist(:,:) = veget(:,:)


    ! Distribution of the vegetation depending on the soil type

!!$    soil_exist(:,:) = zero
!!$    DO ji = 1, kjpindex
!!$       IF(vegtot(ji).GT.min_sechiba) THEN
!!$          soil_exist(ji,:) = mask_soiltile(ji,:)/vegtot(ji)
!!$       ENDIF
!!$    ENDDO
    soil_exist(:,:) = mask_soiltile(:,:)

!!$    veget_exist(:,:) = zero
!!$    DO jv = 1, nvm
!!$       DO ji = 1, kjpindex
!!$          IF(vegtot(ji).GT.min_sechiba) THEN
!!$             veget_exist(ji,jv) = veget(ji,jv)/vegtot(ji)
!!$          ENDIF
!!$       ENDDO
!!$    ENDDO
    veget_exist(:,:) = veget(:,:)

    ! Compute corr_veg_soil 

    corr_veg_soil(:,:,:) = zero

!!$    test_pref(:)=zero
    denom(:) = SUM( veget(:,:) * frac_bare(:,:), dim=2)
!!$    WRITE(numout,*) "denom =",denom
!!$    WRITE(numout,*) "denom/vegtot =",denom/vegtot

    DO jst = 1, nstm
       DO jv = nvm, 1, -1
          
          ! The loop on jst is a simple way to treat the cases where the pref_soil_veg are not fully respected (changing veget)
          ! In the future, this should not be done any more: soiltiles should be allowed to change with veget (LPJ?),
          ! but it implies that we should exchange some water between the different tiles to conserve the water balance
          jst_pref = MODULO(pref_soil_veg(jv)+(jst-2),nstm) + 1
          
          DO ji=1,kjpindex
             corr_veg_soil(ji,jv,jst_pref) = zero
!!$             WRITE(numout,*) 'ji,jv,jst_pref,jst, ',ji,jv,jst_pref,jst, &
!!$                  'corr_veg_soil(ji,jv,jst_pref), veget_exist(ji,jv), soil_exist(ji,jst_pref) ',&
!!$                  corr_veg_soil(ji,jv,jst_pref), veget_exist(ji,jv), soil_exist(ji,jst_pref)

             !for veget distribution used in sechiba via humrel
             IF (soil_exist(ji,jst_pref).GT.min_sechiba) THEN
                corr_veg_soil(ji,jv,jst_pref)=MIN(veget_exist(ji,jv)/soiltile(ji,jst_pref),soil_exist(ji,jst_pref))
!!$                test_pref(ji) = test_pref(ji) + &
!!$                     MIN(veget_exist(ji,jv),soil_exist(ji,jst_pref)*soiltile(ji,jst_pref) ) * frac_bare(ji,jv)
!!$                IF (ji .EQ. 17 .OR. ji .EQ. 24) THEN
!!$                   WRITE(numout,*) 'ji=',ji,' !!'
!!$                   WRITE(numout,*) 'lalo(ji,1:2) = ',lalo(ji,1:2)
!!$                   WRITE(numout,*) 'jst,jst_pref,jv',jst,jst_pref,jv
!!$                   WRITE(numout,*) 'frac_bare',frac_bare(ji,jv)
!!$                   WRITE(numout,*) 'corr_veg_soil',corr_veg_soil(ji,jv,jst_pref)
!!$                   WRITE(numout,*) 'vegtot,veget_exist,soiltile,soil_exist',vegtot(ji),veget_exist(ji,jv), &
!!$                        & soiltile(ji,jst_pref),soil_exist(ji,jst_pref)
!!$                ENDIF
                veget_exist(ji,jv)=MAX(veget_exist(ji,jv)-soil_exist(ji,jst_pref)*soiltile(ji,jst_pref),zero)
                soil_exist(ji,jst_pref)=MAX(soil_exist(ji,jst_pref)-corr_veg_soil(ji,jv,jst_pref),zero)

             ENDIF

!!$             WRITE(numout,*) 'ji,jv,jst_pref,jst, ',ji,jv,jst_pref,jst, &
!!$                  'corr_veg_soil(ji,jv,jst_pref), veget_exist(ji,jv), soil_exist(ji,jst_pref) ',&
!!$                  corr_veg_soil(ji,jv,jst_pref), veget_exist(ji,jv), soil_exist(ji,jst_pref)
          ENDDO
          
       ENDDO
    ENDDO

!!$    WRITE(numout,*) "test_pref =",test_pref
!!$    WRITE(numout,*) "test_pref/denom ="
!!$    DO ji=1,kjpindex
!!$       IF (ABS( denom(ji) ) .GT. zero ) THEN
!!$          WRITE(numout,'(F12.8," ",$)') test_pref(ji)/denom(ji)
!!$       ELSE
!!$          WRITE(numout,'(a3,1X,$)') "INF"
!!$       ENDIF
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$
!!$    WRITE(numout,*) "un-test_pref/denom ="
!!$    DO ji=1,kjpindex
!!$       IF (ABS( denom(ji) ) .GT. zero ) THEN
!!$          WRITE(numout,'(E15.8," ",$)') un-test_pref(ji)/denom(ji)
!!$       ELSE
!!$          WRITE(numout,'(a3,1X,$)') "INF"
!!$       ENDIF
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$
!!$    WRITE(numout,*) "vegtot :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') vegtot(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$
!!$    denom(:)=zero
!!$    DO jst = 1, nstm
!!$       DO jv = 1,nvm
!!$          DO ji=1,kjpindex
!!$             denom(ji) = denom(ji) + corr_veg_soil(ji,jv,jst)
!!$          ENDDO
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "Somme des corr_veg_soil :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
    !
    ! update the corresponding masks
    !
    mask_veget(:,:) = 0
    
    DO jv = 1, nvm
       DO ji = 1, kjpindex
          IF(veget(ji,jv) .GT. min_sechiba) THEN
             mask_veget(ji,jv) = 1
          ENDIF
       END DO
       
       !      print *,'mask: soiltile,mask_soiltile',soiltile(ji,:),mask_soiltile(ji,:)

    END DO
      !
    ! Tout dans cette routine est maintenant certainement obsolete (veget etant constant) en dehors des lignes suivantes:
    frac_bare_ns(:,:) = zero
    tot_frac_bare(:) = zero
    DO jst = 1, nstm
       DO jv = 1, nvm
          DO ji =1, kjpindex
             frac_bare_ns(ji,jst) = frac_bare_ns(ji,jst) + corr_veg_soil(ji,jv,jst) * frac_bare(ji,jv) / vegtot(ji)
!!$             frac_bare_ns(ji,jst) = frac_bare_ns(ji,jst) + corr_veg_soil(ji,jv,jst) * frac_bare(ji,jv)
          ENDDO
       ENDDO
    ENDDO

!!$    denom(:)=zero
!!$    DO jst = 1, nstm
!!$       DO ji=1,kjpindex
!!$          denom(ji) = denom(ji) + frac_bare_ns(ji,jst)
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "Somme des frac_bare_ns :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$    denom(:)=zero
!!$    DO jv = 1,nvm
!!$       DO ji=1,kjpindex
!!$          denom(ji) = denom(ji) + frac_bare(ji,jv)
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "Somme des frac_bare :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
    
    DO jv = 1, nvm
       DO ji =1, kjpindex
          tot_frac_bare(ji) = tot_frac_bare(ji) + veget(ji,jv) * frac_bare(ji,jv)
       ENDDO
    ENDDO

!!$    WRITE(numout,*) "tot_frac_bare :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') tot_frac_bare(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$
!!$    denom(:)=zero
!!$    DO jv = 1, nvm
!!$       DO ji =1, kjpindex
!!$          IF(vegtot(ji).GT.min_sechiba) THEN
!!$             denom(ji) = denom(ji) + veget(ji,jv) * frac_bare(ji,jv) / vegtot(ji)
!!$          ENDIF
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "tot_frac_bare / vegtot :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$
!!$    denom(:)=zero
!!$    DO jst = 1, nstm
!!$       DO ji =1, kjpindex
!!$          denom(ji) = denom(ji) + frac_bare_ns(ji,jst) / tot_frac_bare(ji)
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "frac_bare_ns / tot_frac_bare :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 
!!$    
!!$    denom(:)=zero
!!$    DO jst = 1, nstm
!!$       DO ji =1, kjpindex
!!$          IF(vegtot(ji).GT.min_sechiba) THEN
!!$             denom(ji) = denom(ji) + frac_bare_ns(ji,jst) / tot_frac_bare(ji) / vegtot(ji)
!!$          ELSE
!!$             denom(ji) = denom(ji) + frac_bare_ns(ji,jst) / tot_frac_bare(ji)
!!$          ENDIF
!!$       ENDDO
!!$    ENDDO
!!$    WRITE(numout,*) "frac_bare_ns / tot_frac_bare / vegtot :"
!!$    DO ji=1,kjpindex
!!$       WRITE(numout,'(F12.8,2X,$)') denom(ji)
!!$    ENDDO
!!$    WRITE(numout,*) 

    RETURN
    !
  END SUBROUTINE hydrol_vegupd
  !!
  !! this routine computes the evolution of the surface reservoir (floodplain)
  !!
  SUBROUTINE hydrol_flood (kjpindex, dtradia, vevapflo, flood_frac, flood_res, floodout)
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex 
    ! input fields
    REAL(r_std), INTENT (in)                                 :: dtradia          !! Time step in seconds
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: flood_frac       !! Fraction of floodplains in grid box
    ! modified fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: floodout         !! Flux to take out from floodplains
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: flood_res        !! Floodplains reservoir estimate
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapflo         !! Evaporation over floodplains
    ! local declaration
    INTEGER(i_std)                                          :: ji, jst, jv      !! indices
    REAL(r_std)                                              :: k_m              !! conductivity in the soil
    REAL(r_std), DIMENSION (kjpindex)                        :: temp              !! 

    !- 
    !- 1. Take out vevapflo from the reservoir and transfer the remaining to subsinksoil 
    !-
    DO ji = 1,kjpindex
       temp(ji) = MIN(flood_res(ji), vevapflo(ji))
    ENDDO
    DO ji = 1,kjpindex
       flood_res(ji) = flood_res(ji) - temp(ji)
       subsinksoil(ji) = subsinksoil(ji) + vevapflo(ji) - temp(ji)
       vevapflo(ji) = temp(ji)
    ENDDO

    !- 
    !- 2. Compute the total flux from floodplain floodout (transfered to routing) 
    !-
    DO ji = 1,kjpindex
       floodout(ji) = vevapflo(ji) - flood_frac(ji) * SUM(precisol(ji,:))
    ENDDO

    !-
    !- 3. Discriminate between precip over land and over floodplain
    !-
    DO jv=1, nvm
       DO ji = 1,kjpindex
          precisol(ji,jv) = precisol(ji,jv) * (1 - flood_frac(ji))
       ENDDO
    ENDDO 

    IF (long_print) WRITE (numout,*) ' hydrol_flood done'

  END SUBROUTINE hydrol_flood
  !!
  !! this routine computes soil processes with CWRR scheme
  !!
  SUBROUTINE hydrol_soil (kjpindex, dtradia, veget, soiltile, njsc, reinf_slope, &
       & transpir, vevapnu, evapot, evapot_penm, runoff, drainage, returnflow, reinfiltration, irrigation, &
       & tot_melt, evap_bare_lim,  shumdiag, k_litt, litterhumdiag, humrel,vegstress, drysoil_frac)
    ! 
    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex 
    ! input fields
    REAL(r_std), INTENT (in)                                 :: dtradia          !! Time step in seconds
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)       :: veget            !! Map of vegetation types
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: njsc             !! Indeces of the soiltype 
    REAL(r_std), DIMENSION (kjpindex,nstm), INTENT (in)      :: soiltile         !! Map of soil types
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)        :: transpir         !! transpiration
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)           :: reinf_slope      !! slope coef
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapnu          !! 
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: returnflow       !! Water returning to the deep reservoir
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: reinfiltration   !! Water returning to the top of the soil
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: irrigation       !! Irrigation 
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: evapot           !! 
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: evapot_penm      !! 
    ! modified fields
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: runoff           !! complete runoff
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: drainage         !! complete drainage
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: tot_melt
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: evap_bare_lim    !!
    REAL(r_std), DIMENSION (kjpindex,nbdl), INTENT (out)     :: shumdiag         !! relative soil moisture
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: k_litt           !! litter cond.
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: litterhumdiag    !! litter humidity
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (inout)    :: humrel           !! Relative humidity
    REAL(r_std), DIMENSION (kjpindex, nvm), INTENT(out)      :: vegstress        !! Veg. moisture stress (only for vegetation growth)
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: drysoil_frac     !! Function of the litter humidity, that will be used to compute albedo
    !
    ! local declaration
    !
    INTEGER(i_std)                                 :: ji, jv, jsl, jsl1, jst     !! indices
    REAL(r_std), PARAMETER                         :: frac_mcs = 0.66            !! temporary depth
    REAL(r_std)                                    :: dztmp                      !! temporary depth
    REAL(r_std), DIMENSION(kjpindex)               :: temp                       !! temporary value for fluxes
    REAL(r_std), DIMENSION(kjpindex)               :: tmcold, tmcint
    REAL(r_std), DIMENSION(kjpindex,nslm,nstm)     :: moderwilt
    REAL(r_std), DIMENSION(kjpindex,nslm)          :: mcint                      !! To save mc values for future use
    REAL(r_std), DIMENSION(kjpindex)               :: under_mcr                  !! Allows under residual soil moisture due to evap 
    LOGICAL, DIMENSION(kjpindex)                   :: under_mcr_test             !! test on under residual soil moisture and Mask soil Tile
    LOGICAL, DIMENSION(kjpindex)                   :: under_mcr_test1            !! test on under residual soil moisture
    REAL(r_std), DIMENSION(kjpindex)               :: over_mcs                   !! Allows over saturated soil moisture due to returnflow 
    REAL(r_std), DIMENSION(kjpindex,nstm)          :: evap_bare_lim_ns           !! limitation of bare soil evaporation on each soil column (used to deconvoluate vevapnu)
    REAL(r_std), DIMENSION(kjpindex)               :: deltahum,diff
    LOGICAL(r_std), DIMENSION(kjpindex)            :: test
    REAL(r_std), DIMENSION(kjpindex)               :: tsink
    REAL(r_std), DIMENSION(kjpindex)               :: returnflow_soil
    REAL(r_std), DIMENSION(kjpindex)               :: reinfiltration_soil
    REAL(r_std), DIMENSION(kjpindex)               :: irrigation_soil
    REAL(r_std), DIMENSION(kjpindex)               :: flux_infilt
    !
    !
    returnflow_soil(:) = zero
    reinfiltration_soil(:) = zero
    irrigation_soil(:) = zero
    qflux(:,:,:) = zero
    under_mcr(:) = zero
    over_mcs(:) = zero
    flux_infilt(:) = zero
    !
    ! split 2d variables to 3d variables, per soil type
    !
        CALL hydrol_split_soil (kjpindex, veget, soiltile, vevapnu, transpir, humrel, evap_bare_lim)
    !
    ! Common variables
    !
    DO ji=1,kjpindex
       IF(vegtot(ji).GT.min_sechiba) THEN
          returnflow_soil(ji) = zero
          reinfiltration_soil(ji) = (returnflow(ji) + reinfiltration(ji))/vegtot(ji)
          irrigation_soil(ji) = irrigation(ji)/vegtot(ji)
       ENDIF
    ENDDO

    !
    ! for each soil type
    !
    DO jst=1, nstm
       !
       !- We compute the sum of the sinks for future check-up
       DO ji=1,kjpindex
          tsink(ji) = SUM(rootsink(ji,:,jst))+MAX(ae_ns(ji,jst),zero)+subsinksoil(ji)
       ENDDO
       !
       ! The total moisture content is save for balance checks at the end
       tmcold(:) = tmc(:,jst) 

       ! The moisture content is updated with the evaporation from enerbil and with the reinfiltration fraction from previous timestep
       DO ji = 1, kjpindex
          !- the bare soil evaporation is substracted to the soil moisture profile and first to the water available:  
          temp(ji) = MIN(water2infilt(ji,jst), MAX(ae_ns(ji,jst),zero) + subsinksoil(ji))
       ENDDO
       DO ji = 1, kjpindex
          ! First we substract from the surface
          water2infilt(ji,jst) = water2infilt(ji,jst) - temp(ji)
          ! Then to the rest of the soil
       ENDDO

       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             mc(ji,jsl,jst) = mask_soiltile(ji,jst) * (mc(ji,jsl,jst) &
                  & - (MAX(ae_ns(ji,jst),zero) + subsinksoil(ji) - temp(ji)) / (dpu_max*mille))
          ENDDO
       ENDDO

       !- The value of mc is kept in mcint, used in the flux computation after diffusion:
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             mcint(ji,jsl) = mc(ji,jsl,jst)
          ENDDO
       ENDDO

       DO ji = 1, kjpindex
          tmcint(ji) = dz(2,jst) * ( trois*mcint(ji,1) + mcint(ji,2) )/huit 
       ENDDO

       DO jsl = 2,nslm-1
          DO ji = 1, kjpindex
             tmcint(ji) = tmcint(ji) + dz(jsl,jst) &
                  & * (trois*mcint(ji,jsl)+mcint(ji,jsl-1))/huit &
                  & + dz(jsl+1,jst) * (trois*mcint(ji,jsl)+mcint(ji,jsl+1))/huit
          ENDDO
       ENDDO

       DO ji = 1, kjpindex
          tmcint(ji) = tmcint(ji) + dz(nslm,jst) &
               & * (trois * mcint(ji,nslm) + mcint(ji,nslm-1))/huit
       ENDDO
       
       !- Some initialisation necessary for the diffusion scheme to work
       DO ji = 1, kjpindex
          !- We correct rootsink for first two layers so that it is not too low in the first layer
          v1(ji,jst) = dz(2,jst)/huit * (trois * mc(ji,1,jst)+ mc(ji,2,jst))
          rootsink(ji,2,jst) = rootsink(ji,2,jst) + MAX(rootsink(ji,1,jst)-v1(ji,jst), zero) 
          rootsink(ji,1,jst) = MIN(rootsink(ji,1,jst),v1(ji,jst))
          !- estimate maximum evaporation flux in mm/step, assuming the water is available
          flux(ji) = zero
          IF(vegtot(ji).GT.min_sechiba) THEN
             !- Flux = evapot_penm   if frac_bare_ns > min_sechiba
             !-      = zero          else
             flux(ji) = evapot_penm(ji) * &
                  & AINT(frac_bare_ns(ji,jst)+un-min_sechiba)             
          ENDIF
       ENDDO

       ! Then we prepare the infiltration (first the irrigation and below mcr fill up)
       DO ji = 1, kjpindex
          !- We add the irrigation to the soil (irrigation is put so that it does not runoff)
          !- by filling the layers from top to bottom (same for reinfiltration) - this is done by smooth below
!          mc(ji,1,jst) = mc(ji,1,jst) + irrigation_soil(ji) + reinfiltration_soil(ji) * deux / dz(2,jst)
!MG d apres Tristan le 17/07/08
          mc(ji,1,jst) = mc(ji,1,jst) + (irrigation_soil(ji) + reinfiltration_soil(ji)) * deux / dz(2,jst)
       ENDDO

       CALL hydrol_soil_smooth(kjpindex,jst, njsc, under_mcr, over_mcs)

       DO ji = 1, kjpindex
          ! Initialise the flux to be infiltrated 
          flux_infilt(ji) = water2infilt(ji,jst) + precisol_ns(ji,jst) 
          !- The incoming flux is also first dedicated to fill the soil up to mcr (in case needed)
          temp(ji) = MIN(flux_infilt(ji),under_mcr(ji))
       ENDDO
       DO ji = 1, kjpindex
          flux_infilt(ji) = flux_infilt(ji) - temp(ji)
          under_mcr(ji) = under_mcr(ji) - temp(ji)
       ENDDO

       !- coefficients are computed for the profile of mc before rainfall infiltration:
       CALL hydrol_soil_coef(kjpindex,jst,njsc)

       ! Then the infiltration by precipitation is computed 
       CALL hydrol_soil_infilt(kjpindex, jst, dtradia, njsc, flux_infilt, over_mcs)

       !- module to implement dublin model for one time-step
       !-	gravity drainage as lower boundary layer
       !- m.bruen, CWRR, ucd. 
       !
       !-step3: matrix resolution
       !-calcul of the matrix coefficients

       !- coefficient are recomputed for the new profile of mc:
       CALL hydrol_soil_coef(kjpindex,jst,njsc)

       !- Set the values for diffusion scheme
       CALL hydrol_soil_setup(kjpindex,jst,dtradia)

       resolv(:) = .FALSE.
       under_mcr_test1(:)=(under_mcr(:).LT.min_sechiba)
       under_mcr_test(:)=(mask_soiltile(:,jst) .GT. 0).AND.under_mcr_test1(:)
       DO ji = 1, kjpindex
          ! We only run the scheme in case we are not under mcr after precip and various reinfiltrations and not over mcs
          IF (under_mcr_test(ji)) THEN
             IF (over_mcs(ji).LT.min_sechiba) THEN
                resolv(ji)=.TRUE.
             ENDIF
          ENDIF
       ENDDO

       DO ji = 1, kjpindex
          ! We only run the scheme in case we are not under mcr after precip and various reinfiltrations and not over mcs
          IF (under_mcr_test(ji)) THEN
             IF (.NOT. resolv(ji)) THEN
                ! In oversaturated case, we first take the evaporation from the outgoing water (the rest will be taken from the soil)
                over_mcs(ji) = over_mcs(ji) - flux(ji)
             ENDIF
          ENDIF
       ENDDO
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             ! We only run the scheme in case we are not under mcr after precip and various reinfiltrations and not over mcs
             IF (under_mcr_test(ji)) THEN
                IF (.NOT. resolv(ji)) THEN
                   ! In oversaturated case, we first take the evaporation from the outgoing water (the rest will be taken from the soil)
                   over_mcs(ji) = over_mcs(ji) - rootsink(ji,jsl,jst)
                ENDIF
             ENDIF
          ENDDO
       ENDDO
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             ! We only run the scheme in case we are not under mcr after precip and various reinfiltrations and not over mcs
             IF (mask_soiltile(ji,jst) .GT. 0) THEN
                IF (.NOT. under_mcr_test1(ji)) THEN
                   ! In under residual case, we equally spread the transpiration over the layers
                   under_mcr(ji) = under_mcr(ji) + rootsink(ji,jsl,jst)
                ENDIF
             ENDIF
          ENDDO
       ENDDO
       
       DO ji = 1, kjpindex
          ! We directly use the value of k (a and b are only used un hydrol_soil_coef to compute k)
          !-        First layer
          tmat(ji,1,1) = zero
          tmat(ji,1,2) = f(ji,1)
          tmat(ji,1,3) = g1(ji,1)
          rhs(ji,1)    = fp(ji,1) * mc(ji,1,jst) + gp(ji,1)*mc(ji,2,jst) &
               &  - flux(ji) - SQRT(k(ji,1)*k(ji,2)) *(dtradia/one_day) - rootsink(ji,1,jst)
       ENDDO
          
          !-	soil body
       DO jsl=2, nslm-1
          DO ji = 1, kjpindex
             tmat(ji,jsl,1) = e(ji,jsl)
             tmat(ji,jsl,2) = f(ji,jsl)
             tmat(ji,jsl,3) = g1(ji,jsl)
             rhs(ji,jsl) = ep(ji,jsl)*mc(ji,jsl-1,jst) + fp(ji,jsl)*mc(ji,jsl,jst) &
                  & +  gp(ji,jsl) * mc(ji,jsl+1,jst) & 
                  & + (SQRT(k(ji,jsl-1)*k(ji,jsl)) - SQRT(k(ji,jsl)*k(ji,jsl+1))) * (dtradia/one_day) & 
                  & - rootsink(ji,jsl,jst) 
          ENDDO
       ENDDO
       
       DO ji = 1, kjpindex
          !-        Last layer 
          jsl=nslm
          tmat(ji,jsl,1) = e(ji,jsl)
          tmat(ji,jsl,2) = f(ji,jsl)
          tmat(ji,jsl,3) = zero
          rhs(ji,jsl) = ep(ji,jsl)*mc(ji,jsl-1,jst) + fp(ji,jsl)*mc(ji,jsl,jst) &
               & + (SQRT(k(ji,jsl-1)*k(ji,jsl)) - k(ji,jsl)) * (dtradia/one_day) & 
               & - rootsink(ji,jsl,jst)
       ENDDO

          !- store the equations in case needed again
       DO jsl=1,nslm
          DO ji = 1, kjpindex
             srhs(ji,jsl) = rhs(ji,jsl)
             stmat(ji,jsl,1) = tmat(ji,jsl,1)
             stmat(ji,jsl,2) = tmat(ji,jsl,2)
             stmat(ji,jsl,3) = tmat(ji,jsl,3) 
          ENDDO
       ENDDO

       !
       !- step 4 : solve equations assuming atmosphere limiting (sufficient soil moisture for Evapot)
       !-

       CALL hydrol_soil_tridiag(kjpindex,jst)
       !	
       !- step 5 : check if really atmosphere limiting
       !-
       DO ji = 1, kjpindex
          !
          !- Prepare to rerun in case of under residual with evaporation 
          !-
!          resolv(ji) = (mc(ji,1,jst).LT.(mcr(njsc(ji))*0.9).AND.flux(ji).GT.min_sechiba)
!MG erreur!!!
          resolv(ji) = (mc(ji,1,jst).LT.(mcr(njsc(ji))).AND.flux(ji).GT.min_sechiba)
       ENDDO
       ! Reset the coefficient for diffusion (only used if resolv(ji) = .TRUE.)
       DO jsl=1,nslm
          !- The new condition is to put the upper layer at residual soil moisture
          DO ji = 1, kjpindex
             rhs(ji,jsl) = srhs(ji,jsl)
             tmat(ji,jsl,1) = stmat(ji,jsl,1)
             tmat(ji,jsl,2) = stmat(ji,jsl,2)
             tmat(ji,jsl,3) = stmat(ji,jsl,3)
          END DO
       ENDDO
       !
       DO ji = 1, kjpindex
          tmat(ji,1,2) = un
          tmat(ji,1,3) = zero

!          rhs(ji,1) = mcr(njsc(ji))*0.9
!MG erreur!!!
          rhs(ji,1) = mcr(njsc(ji))
       ENDDO
       !
       !- step 6 : resolve the equations with new boundary conditions if necessary
       !-

       CALL hydrol_soil_tridiag(kjpindex,jst)
       !-
       !- step 6.5 : initialize qflux at bottom of diffusion and avoid over saturated or under residual soil moisture 
       !-

       DO ji = 1, kjpindex

          dr_ns(ji,jst)=zero
          jsl=nslm
          IF (under_mcr(ji).LT.min_sechiba) THEN
             dr_ns(ji,jst) = mask_soiltile(ji,jst)*k(ji,jsl) * (dtradia/one_day)
          ENDIF

       ENDDO

       ! Finally, for soil that are under-residual, we just equally distribute the lack of water
       ! For over-saturated ones, we do the same if the outgoing flux was not sufficient to satisfy evapot
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             mc(ji,jsl,jst) = mc(ji,jsl,jst) - under_mcr(ji) / (dpu_max*mille)
             mc(ji,jsl,jst) = mask_soiltile(ji,jst) * mc(ji,jsl,jst) + MIN(zero, over_mcs(ji)) / (dpu_max*mille)
          ENDDO
       ENDDO

       !
       !- step 7 : close the water balance
       !
       DO ji = 1, kjpindex
          tmc(ji,jst) = dz(2,jst) * ( trois*mc(ji,1,jst) + mc(ji,2,jst) )/huit 
       ENDDO
          
       DO jsl = 2,nslm-1
          DO ji = 1, kjpindex
             tmc(ji,jst) = tmc(ji,jst) + dz(jsl,jst) &
                  & * (trois*mc(ji,jsl,jst)+mc(ji,jsl-1,jst))/huit &
                  & + dz(jsl+1,jst) * (trois*mc(ji,jsl,jst)+mc(ji,jsl+1,jst))/huit
          ENDDO
       ENDDO

       DO ji = 1, kjpindex
          tmc(ji,jst) = tmc(ji,jst) + dz(nslm,jst) &
               & * (trois * mc(ji,nslm,jst) + mc(ji,nslm-1,jst))/huit
       ENDDO

       DO ji = 1, kjpindex
          qflux00(ji,jst) = mask_soiltile(ji,jst) * &
               & (MIN(tmcs(ji,jst),tmc(ji,jst))-tmcint(ji)+SUM(rootsink(ji,:,jst))+dr_ns(ji,jst)-returnflow_soil(ji))

          !
          ! deduction of ae_ns and ru_ns:
          ! ae_ns+ru_ns=precisol_ns+irrigation-q0  
          !
          ae_ns(ji,jst) = MAX(MIN((precisol_ns(ji,jst)+irrigation_soil(ji)+reinfiltration_soil(ji) & ! (returnflow introduit par le haut)
               & +water2infilt(ji,jst)-qflux00(ji,jst)) ,evapot_penm(ji)),zero) * mask_soiltile(ji,jst)
          ru_ns(ji,jst) = (precisol_ns(ji,jst)+irrigation_soil(ji)+reinfiltration_soil(ji) &
               & +water2infilt(ji,jst)-qflux00(ji,jst)-ae_ns(ji,jst)) * mask_soiltile(ji,jst) 
       ENDDO

       !
       ! Special treatment for the unstable cases when boundary condition mc1=mcr leads to negative runoff
       !
       IF (long_print) THEN
          DO ji = 1, kjpindex
             IF (ru_ns(ji,jst) .LT.-min_sechiba) THEN
                WRITE (numout,*) 'Negative runoff corrected', ji,jst,ru_ns(ji,jst), mc(ji,1,jst), tmc(ji,jst)
             ENDIF
          ENDDO
       ENDIF

       DO ji = 1, kjpindex
          temp(ji) = MIN(ru_ns(ji,jst),zero)
       ENDDO
       DO ji = 1, kjpindex
          ru_ns(ji,jst) = ru_ns(ji,jst) - temp(ji)
          ! We correct this by taking water from the whole soil
          qflux00(ji,jst) = qflux00(ji,jst) + temp(ji)
       ENDDO
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             mc(ji,jsl,jst) = mc(ji,jsl,jst) + temp(ji) / (dpu_max*mille)
          ENDDO
       ENDDO

       ! Avoid under-precision value for the 3 outward flux
       DO ji = 1, kjpindex  
          IF (ABS(ae_ns(ji,jst)).LT.min_sechiba) THEN
             ae_ns(ji,jst) = zero
          ENDIF

          IF(ABS(ru_ns(ji,jst)).LT.min_sechiba) THEN
             ru_ns(ji,jst) = zero
          ENDIF

          IF(ABS(dr_ns(ji,jst)).LT.min_sechiba) THEN
             dr_ns(ji,jst) = zero
          ENDIF
       ENDDO

       ! Then compute the temporary surface water and correct the outgoing runoff
       IF ( .NOT. doponds ) THEN
          DO ji = 1, kjpindex
             water2infilt(ji,jst) = reinf_slope(ji) * ru_ns(ji,jst)
          ENDDO
       ELSE
          DO ji = 1, kjpindex           
             water2infilt(ji,jst) = zero
          ENDDO
       ENDIF
       !
       DO ji = 1, kjpindex           
          ru_ns(ji,jst) = ru_ns(ji,jst) - water2infilt(ji,jst)
          water2infilt(ji,jst) = water2infilt(ji,jst) + ae_ns(ji,jst)
       ENDDO


       CALL hydrol_soil_smooth(kjpindex, jst, njsc, under_mcr, over_mcs)
!MG d apres Tristan le 17/07/08
       ! Finally, for soil that are under-residual, we just equally distribute the lack of water
       ! For over-saturated ones, we do the same if the outgoing flux was not sufficient to satisfy evapot
       DO jsl = 1, nslm
          DO ji = 1, kjpindex
             mc(ji,jsl,jst) = mc(ji,jsl,jst) - under_mcr(ji) / (dpu_max*mille)
             mc(ji,jsl,jst) = mask_soiltile(ji,jst) * mc(ji,jsl,jst) + MIN(zero, over_mcs(ji)) / (dpu_max*mille)
          ENDDO
       ENDDO

       ! Optional computation of the fluxes 
       IF ( check_cwrr ) THEN
          CALL hydrol_soil_flux(kjpindex,jst,mcint,returnflow_soil)
       ENDIF

       !
       !- step8: we make some useful output
       !- Total soil moisture, soil moisture at litter levels, soil wetness...
       ! 

       !-total soil moisture:
       DO ji = 1, kjpindex
          tmc(ji,jst)= dz(2,jst) * (trois*mc(ji,1,jst) + mc(ji,2,jst))/huit
       ENDDO

       DO jsl=2,nslm-1
          DO ji = 1, kjpindex
             tmc(ji,jst) = tmc(ji,jst) + dz(jsl,jst) * ( trois*mc(ji,jsl,jst) + mc(ji,jsl-1,jst))/huit &
                  & + dz(jsl+1,jst)*(trois*mc(ji,jsl,jst) + mc(ji,jsl+1,jst))/huit
          ENDDO
       END DO

       DO ji = 1, kjpindex
          tmc(ji,jst) = tmc(ji,jst) +  dz(nslm,jst) * (trois * mc(ji,nslm,jst) + mc(ji,nslm-1,jst))/huit
          tmc(ji,jst) = tmc(ji,jst) + water2infilt(ji,jst)
       ENDDO

       ! the litter is the 4 top levels of the soil
       ! we compute various field of soil moisture for the litter (used for stomate and for albedo)

       DO ji = 1, kjpindex
          tmc_litter(ji,jst) = dz(2,jst) * ( trois*mc(ji,1,jst)+ mc(ji,2,jst))/huit
          tmc_litter(ji,jst) = tmc_litter(ji,jst) 
       ENDDO

       ! sum from level 1 to 4

       DO jsl=2,4
          DO ji = 1, kjpindex
             tmc_litter(ji,jst) = tmc_litter(ji,jst) + dz(jsl,jst) * & 
                  & ( trois*mc(ji,jsl,jst) + mc(ji,jsl-1,jst))/huit &
                  & + dz(jsl+1,jst)*(trois*mc(ji,jsl,jst) + mc(ji,jsl+1,jst))/huit
          END DO
       ENDDO

       ! subsequent calcul of soil_wet_litter (tmc-tmcw)/(tmcf-tmcw)

       DO ji = 1, kjpindex
          soil_wet_litter(ji,jst) = MIN(un, MAX(zero,&
               & (tmc_litter(ji,jst)-tmc_litter_wilt(ji,jst)) / &
               & (tmc_litter_field(ji,jst)-tmc_litter_wilt(ji,jst)) ))
       ENDDO

       ! Soil wetness profiles (mc-mcw)/(mcs-mcw)
       ! soil_wet is the ratio of soil moisture to available soil moisture for plant
       ! (ie soil moisture at saturation minus soil moisture at wilting point).

       DO ji = 1, kjpindex
          soil_wet(ji,1,jst) = MIN(un, MAX(zero,&
               & (trois*mc(ji,1,jst) + mc(ji,2,jst) - quatre*mcw(njsc(ji)))&
               & /(quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
          humrelv(ji,1,jst) = zero
       ENDDO

       DO jsl=2,nslm-1
          DO ji = 1, kjpindex
             soil_wet(ji,jsl,jst) = MIN(un, MAX(zero,&
                  & (trois*mc(ji,jsl,jst) + & 
                  & mc(ji,jsl-1,jst) *(dz(jsl,jst)/(dz(jsl,jst)+dz(jsl+1,jst))) &
                  & + mc(ji,jsl+1,jst)*(dz(jsl+1,jst)/(dz(jsl,jst)+dz(jsl+1,jst))) &
                  & - quatre*mcw(njsc(ji))) / (quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
          END DO
       ENDDO

       DO ji = 1, kjpindex
          soil_wet(ji,nslm,jst) = MIN(un, MAX(zero,&
               & (trois*mc(ji,nslm,jst) &
               & + mc(ji,nslm-1,jst)-quatre*mcw(njsc(ji)))/(quatre*(mcs(njsc(ji))-mcw(njsc(ji)))) ))
       ENDDO

       !
       !- step8: we make the outputs for sechiba:
       !-we compute the moderation of transpiration due to wilting point:
       ! moderwilt is a factor which is zero if soil moisture is below the wilting point
       ! and is un if soil moisture is above the wilting point.


       DO jsl=1,nslm
          DO ji = 1, kjpindex
             moderwilt(ji,jsl,jst) = INT( MAX(soil_wet(ji,jsl,jst), zero) + un - min_sechiba )
          END DO
       ENDDO

       !- we compute the new humrelv to use in sechiba:
       !- loop on each vegetation type

       humrelv(:,1,jst) = zero   

          !- calcul of us for each layer and vegetation type.

       DO jv = 2,nvm
          DO ji = 1, kjpindex
             !- Here we make the assumption that roots do not take water from the 1st layer. 
             !- Comment the us=0 if you want to change this.
!             us(ji,jv,jst,1) = moderwilt(ji,1,jst)*MIN(un,((trois*mc(ji,1,jst) + mc(ji,2,jst)) &
!                  & /(quatre*mcs(jst)*pcent(jst))) )* (un-EXP(-humcste(jv)*dz(2,jst)/mille/deux)) &
!                  & /(un-EXP(-humcste(jv)*zz(nslm,jst)/mille))
             us(ji,jv,jst,1) = zero
             humrelv(ji,jv,jst) = MAX(us(ji,jv,jst,1),zero)
          ENDDO
       ENDDO

       DO jsl = 2,nslm-1
          DO jv = 2, nvm
             DO ji = 1, kjpindex
                ! us is computed with a SQRT in order for it to grow more rapidly with soil moisture.
                ! it is not essential
                us(ji,jv,jst,jsl) =moderwilt(ji,jsl,jst)* &
                     & MIN( un, &
                     & SQRT((trois*mc(ji,jsl,jst)+ &
                     & mc(ji,jsl-1,jst)*(dz(jsl,jst)/(dz(jsl,jst)+dz(jsl+1,jst)))+ &
                     & mc(ji,jsl+1,jst)*(dz(jsl+1,jst)/(dz(jsl,jst)+dz(jsl+1,jst)))) &
                     & /(quatre*mcs(njsc(ji))*pcent(njsc(ji)))) )* &
                     & nroot(jv,jst,jsl)

                us(ji,jv,jst,jsl) = MAX(us (ji,jv,jst,jsl), zero)
                humrelv(ji,jv,jst) = MAX((humrelv(ji,jv,jst) + us(ji,jv,jst,jsl)),zero)
             END DO
          ENDDO
       ENDDO

       DO jv = 2, nvm
          DO ji = 1, kjpindex
             us(ji,jv,jst,nslm) = moderwilt(ji,nslm,jst)*  &
                  & MIN(un, &
                  & SQRT((trois*mc(ji,nslm,jst) + mc(ji,nslm-1,jst))  &
                  &                    / (quatre*mcs(njsc(ji))*pcent(njsc(ji)))) ) * &
                  & nroot(jv,jst,nslm)

             us(ji,jv,jst,nslm) = MAX(us(ji,jv,jst,nslm), zero)
             humrelv(ji,jv,jst) = MAX(zero,MIN(un, humrelv(ji,jv,jst) + us(ji,jv,jst,nslm)))
             vegstressv(ji,jv,jst) = humrelv(ji,jv,jst)
          END DO
       END DO

       DO jv = 2, nvm
          DO ji = 1, kjpindex
             IF (corr_veg_soil(ji,jv,jst) .LT. min_sechiba) THEN
                humrelv(ji,jv,jst) = zero  
             ENDIF
          END DO
       END DO

       !before closing the soil water, we check the water balance of soil

       IF(check_cwrr) THEN
          DO ji = 1,kjpindex
             
             deltahum(ji) = (tmc(ji,jst) - tmcold(ji))
             diff(ji)     = precisol_ns(ji,jst)-ru_ns(ji,jst)-dr_ns(ji,jst)-tsink(ji) &
                  & + irrigation_soil(ji) + returnflow_soil(ji) + reinfiltration_soil(ji)

             test(ji) = (abs(deltahum(ji)-diff(ji))*mask_soiltile(ji,jst) .GT. allowed_err)
          ENDDO

          DO ji = 1,kjpindex
             IF(test(ji)) THEN
                
                WRITE (numout,*)'CWRR pat: bilan non nul',ji,jst,njsc(ji),deltahum(ji)-diff(ji)
                WRITE (numout,*)'tmc,tmcold,diff',tmc(ji,jst),tmcold(ji),deltahum(ji)
                WRITE (numout,*)'evapot,evapot_penm,ae_ns',evapot(ji),evapot_penm(ji),ae_ns(ji,jst)
                WRITE (numout,*)'flux,ru_ns,qdrain,tsink,q0,precisol',flux(ji),ru_ns(ji,jst), &
                     &      dr_ns(ji,jst),tsink(ji),qflux00(ji,jst),precisol_ns(ji,jst)
                WRITE (numout,*)'water2infilt',water2infilt(ji,jst)
                WRITE (numout,*)'soiltile',soiltile(ji,jst)
                WRITE (numout,*)'irrigation, returnflow, reinfiltration', &
                     & irrigation_soil(ji),returnflow_soil(ji),reinfiltration_soil(ji)
                WRITE (numout,*)'mc',mc(ji,:,jst)
                WRITE (numout,*)'qflux',qflux(ji,:,jst)
                WRITE (numout,*)'veget', veget(ji,:)
                WRITE (numout,*)'k', k(ji,:)
                waterbal_error=.TRUE.
                CALL ipslerr(2, 'hydrol_soil', 'We will STOP after hydrol_waterbal.',&
                     & 'CWRR water balance check','')
                
             ENDIF
          ENDDO

          DO ji = 1,kjpindex
             
             IF(MINVAL(mc(ji,:,jst)).LT.-min_sechiba) THEN
                WRITE (numout,*)'CWRR MC NEGATIVE', &
                     & ji,lalo(ji,:),MINLOC(mc(ji,:,jst)),jst,mc(ji,:,jst)
                WRITE (numout,*)'evapot,evapot_penm,ae_ns',evapot(ji),evapot_penm(ji),ae_ns(ji,jst)
                WRITE (numout,*)'flux,ru_ns,qdrain,tsink,q0,precisol',flux(ji),ru_ns(ji,jst), &
                     &      dr_ns(ji,jst),tsink(ji),qflux00(ji,jst),precisol_ns(ji,jst)
                WRITE (numout,*)'water2infilt',water2infilt(ji,jst)
                WRITE (numout,*)'soiltile',soiltile(ji,jst)
                WRITE (numout,*)'irrigation, returnflow, reinfiltration', &
                     & irrigation_soil(ji),returnflow_soil(ji),reinfiltration_soil(ji)
                WRITE (numout,*)'mc',mc(ji,:,jst)
                WRITE (numout,*)'qflux',qflux(ji,:,jst)
                WRITE (numout,*)'veget', veget(ji,:)
                WRITE (numout,*)'k', k(ji,:)
                WRITE (numout,*)'soiltile',soiltile(ji,jst)
                waterbal_error=.TRUE.
                CALL ipslerr(2, 'hydrol_soil', 'We will STOP after hydrol_waterbal.',&
                     & 'CWRR MC NEGATIVE','')
             ENDIF
          END DO

          DO ji=1,kjpindex
             IF (ru_ns(ji,jst)*soiltile(ji,jst).LT.-min_sechiba) THEN
                WRITE (numout,*) 'Negative runoff', ji,jst, mask_soiltile(ji,jst) 
                WRITE (numout,*) 'mc1, mc2', mc(ji,1,jst), mc(ji,2,jst)
                WRITE (numout,*) 'mcint1, mcint2', mcint(ji,1), mcint(ji,2)
                WRITE (numout,*) 'qflux1, flux', qflux(ji,nslm,jst), flux(ji)
                WRITE (numout,*) 'over_mcs, under_mcr, test', &
                     & over_mcs(ji), under_mcr(ji), tmc(ji,jst)-tmcint(ji)+qflux(ji,nslm,jst)+SUM(rootsink(ji,:,jst))
                WRITE (numout,*) 'mc', mc(ji,:,jst)
                WRITE (numout,*) 'mcint', mcint(ji,:)
                WRITE (numout,*) 'qflux', qflux(ji,:,jst)
                WRITE (numout,*) 'rootsink1,evapot_penm,vegtot', rootsink(ji,1,jst), evapot_penm(ji), vegtot(ji)
                WRITE (numout,*) 'ae_ns, tsink, returnflow, reinfiltration, precisol_ns, irrigation, qflux0, ru_ns', &
                     & ae_ns(ji,jst), tsink(ji), returnflow_soil(ji), reinfiltration_soil(ji), &
                     & precisol_ns(ji,jst), irrigation_soil(ji), qflux00(ji,jst), ru_ns(ji,jst)
                waterbal_error=.TRUE.
                CALL ipslerr(2, 'hydrol_soil', 'We will STOP after hydrol_waterbal.',&
                     & 'Negative runoff, non-saturated soil','')
             ENDIF
          ENDDO
       ENDIF

       IF (long_print) WRITE (numout,*) ' hydrol_soil done for jst =', jst     

    END DO  ! end of loop on soiltile

    !
    ! sum 3d variables into 2d variables
    !
    CALL hydrol_diag_soil (kjpindex, veget, soiltile, njsc, runoff, drainage, &
         & evap_bare_lim, evapot, vevapnu, returnflow, reinfiltration, irrigation, &
         & shumdiag, k_litt, litterhumdiag, humrel, vegstress, drysoil_frac,tot_melt)
    RETURN

  END SUBROUTINE hydrol_soil

  SUBROUTINE hydrol_soil_infilt(kjpindex, ins, dtradia, njsc, flux_infilt, over_mcs)
    ! GLOBAL (in or inout)
    INTEGER(i_std), INTENT(in)                          :: kjpindex        !! Domain size
    REAL(r_std), INTENT (in)                            :: dtradia         !! Time step in seconds
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)    :: njsc            !! Indeces of the soiltile 
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)      :: flux_infilt     !! Water to infiltrate
    REAL(r_std), DIMENSION (kjpindex), INTENT (inout)   :: over_mcs        !! Over saturation
    ! LOCAL DECLARATIONS
    INTEGER(i_std)                                      :: ji, jv, jsl, ins, i !! Indeces
    REAL(r_std)                                         :: m               !! 
    REAL(r_std), DIMENSION (kjpindex)                   :: wat_inf_pot     !! infiltrable water in the layer
    REAL(r_std), DIMENSION (kjpindex)                   :: wat_inf         !! infiltrated water in the layer
    REAL(r_std), DIMENSION (kjpindex)                   :: dt_tmp          !! time remaining before the end of the time step
    REAL(r_std)                                         :: dt_inf          !! the time it takes to complete the infiltration in the layer
    REAL(r_std)                                         :: k_m             !! the mean conductivity used for the saturated front
    REAL(r_std)                                         :: infilt_tmp      !! infiltration rate for the considered layer 
    REAL(r_std), DIMENSION (kjpindex)                   :: infilt_tot      !! total infiltration 
    REAL(r_std), DIMENSION (kjpindex)                   :: flux_tmp        !! rate at which precip hits the ground

    ! If data (or coupling with GCM) was available, a parameterization for subgrid rainfall could be performed

    DO ji = 1, kjpindex
       !-
       !- 1. First layer
       !-
       ! First we fill up the first layer (about 1mm) without any resistance and quasi-immediately
       wat_inf_pot(ji) = (mcs(njsc(ji))-mc(ji,1,ins)) * dz(2,ins) / deux
       wat_inf(ji) = MIN(wat_inf_pot(ji), flux_infilt(ji))
       mc(ji,1,ins) = mc(ji,1,ins) + wat_inf(ji) * deux / dz(2,ins)

    ENDDO

    !-
    !- 2. Infiltration layer by layer 
    !-
    ! Initialize a countdown for infiltration during the time-step and the value of potential runoff
    dt_tmp(:) = dtradia / one_day
    infilt_tot(:) = zero
    ! Compute the rate at which water will try to infiltrate each layer
    flux_tmp(:) = (flux_infilt(:)-wat_inf(:)) / dt_tmp(:)
    DO jsl = 2, nslm-1
       DO ji = 1, kjpindex
          ! Infiltrability of each layer if under a saturated one
          ! This is computed by an simple arithmetic average because 
          ! the time step (30min) is not appropriate for a geometric average (advised by Haverkamp and Vauclin)
          k_m = (k(ji,jsl) + ks(njsc(ji))*kfact(jsl-1,njsc(ji))*kfact_root(ji,jsl,ins)) / deux 

          !- We compute the mean rate at which water actually infiltrate:
          !- Subgrid: Exponential distribution of  k around i_m, but average p directly used 
          infilt_tmp = k_m * (un - EXP(- flux_tmp(ji) / k_m)) 

          ! From which we deduce the time it takes to fill up the layer or to end the time step...
          wat_inf_pot(ji) = (mcs(njsc(ji))-mc(ji,jsl,ins)) * (dz(jsl,ins) + dz(jsl+1,ins)) / deux
          dt_inf =  MIN(wat_inf_pot(ji) / (infilt_tmp + min_sechiba), dt_tmp(ji))

          ! ...and the water that enters the layer...
          wat_inf(ji) = dt_inf * (infilt_tmp)          
          ! ...obviously the moisture content...
          mc(ji,jsl,ins) = mc(ji,jsl,ins) + &
               & wat_inf(ji) * deux / (dz(jsl,ins) + dz(jsl+1,ins))
          ! ...the time remaining before the next time step
          dt_tmp(ji) = dt_tmp(ji) - dt_inf
          ! ...and finally the infilt_tot (which is just used to check if there is a problem, below) 
          infilt_tot(ji) = infilt_tot(ji) + infilt_tmp * dt_inf

       ENDDO
    ENDDO

    ! Register the cases where the soil is saturated
    DO ji = 1, kjpindex
       IF (dt_tmp(ji) .GT. min_sechiba .AND. flux_tmp(ji) .GT. min_sechiba) THEN
          over_mcs(ji) = over_mcs(ji) + dt_tmp(ji) * flux_tmp(ji)
       ENDIF
    ENDDO

    DO ji = 1, kjpindex
       IF (infilt_tot(ji) .LT. -min_sechiba .OR. infilt_tot(ji) .GT. flux_infilt(ji) + min_sechiba) THEN
          WRITE (numout,*) 'Error in the calculation of infilt tot', infilt_tot(ji)
          WRITE (numout,*) 'k, ji, jst, mc', k(ji,1:2), ji, ins, mc(ji,1,ins)
          waterbal_error=.TRUE.
          CALL ipslerr(3, 'hydrol_soil_infilt', 'We will STOP after hydrol_soil_infilt.','','')
       ENDIF
    ENDDO

    RETURN
  END SUBROUTINE hydrol_soil_infilt

  SUBROUTINE hydrol_soil_smooth(kjpindex, ins, njsc, under_mcr, over_mcs)

    !- smooth soil moisture values:
    !- avoid over-saturation or under-residual values

    !- arguments
    INTEGER(i_std), INTENT(in)                         :: kjpindex        !! Domain size
    INTEGER(i_std), INTENT(in)                         :: ins             !! number of soil type
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: njsc            !! Indeces of the soiltile 
    REAL(r_std), DIMENSION(kjpindex), INTENT(out)     :: under_mcr       !! Allows under residual soil moisture due to evap 
    REAL(r_std), DIMENSION(kjpindex), INTENT(out)     :: over_mcs       !! Allows over saturated soil moisture due to returnflow 

    !  -- variables locales

    INTEGER(i_std)                      :: ji,jsl
    REAL(r_std)                          :: excess

       
    !-
    !- Avoid over-saturation values
    !-
    
    !- in case of over-saturation we put the water where it is possible
    DO jsl = 1, nslm-2
       DO ji=1, kjpindex
          excess = MAX(mc(ji,jsl,ins)-mcs(njsc(ji)),zero)
          mc(ji,jsl,ins) = mc(ji,jsl,ins) - excess
          mc(ji,jsl+1,ins) = mc(ji,jsl+1,ins) + excess * &
               &  (dz(jsl,ins)+dz(jsl+1,ins))/(dz(jsl+1,ins)+dz(jsl+2,ins))
       ENDDO
    ENDDO

    jsl = nslm-1
    DO ji=1, kjpindex
       excess = MAX(mc(ji,jsl,ins)-mcs(njsc(ji)),zero)
       mc(ji,jsl,ins) = mc(ji,jsl,ins) - excess
       mc(ji,jsl+1,ins) = mc(ji,jsl+1,ins) + excess * &
            &  (dz(jsl,ins)+dz(jsl+1,ins))/dz(jsl+1,ins)
    ENDDO

    jsl = nslm
    DO ji=1, kjpindex
       excess = MAX(mc(ji,jsl,ins)-mcs(njsc(ji)),zero)
       mc(ji,jsl,ins) = mc(ji,jsl,ins) - excess
       mc(ji,jsl-1,ins) = mc(ji,jsl-1,ins) + excess * &
            &  dz(jsl,ins)/(dz(jsl-1,ins)+dz(jsl,ins))
    ENDDO

    DO jsl = nslm-1,2,-1
       DO ji=1, kjpindex
          excess = MAX(mc(ji,jsl,ins)-mcs(njsc(ji)),zero)
          mc(ji,jsl,ins) = mc(ji,jsl,ins) - excess
          mc(ji,jsl-1,ins) = mc(ji,jsl-1,ins) + excess * &
               &  (dz(jsl,ins)+dz(jsl+1,ins))/(dz(jsl-1,ins)+dz(jsl,ins))
       ENDDO
    ENDDO

    DO ji=1, kjpindex
       excess = mask_soiltile(ji,ins) * MAX(mc(ji,1,ins)-mcs(njsc(ji)),zero)
       mc(ji,1,ins) = mc(ji,1,ins) - excess

       over_mcs(ji) = excess * dz(2,ins)/2  
    ENDDO

    !-
    !- Avoid below residual values
    !-
       
    ! Smooth the profile to avoid negative values of punctual soil moisture
    DO jsl = 1,nslm-2
       DO ji=1, kjpindex
          excess = MAX(mcr(njsc(ji))-mc(ji,jsl,ins),zero)
          mc(ji,jsl,ins) = mc(ji,jsl,ins) + excess
          mc(ji,jsl+1,ins) = mc(ji,jsl+1,ins) - excess * &
               &  (dz(jsl,ins)+dz(jsl+1,ins))/(dz(jsl+1,ins)+dz(jsl+2,ins))
       ENDDO
    ENDDO

    jsl = nslm-1
    DO ji=1, kjpindex
       excess = MAX(mcr(njsc(ji))-mc(ji,jsl,ins),zero)
       mc(ji,jsl,ins) = mc(ji,jsl,ins) + excess
       mc(ji,jsl+1,ins) = mc(ji,jsl+1,ins) - excess * &
            &  (dz(jsl,ins)+dz(jsl+1,ins))/dz(jsl+1,ins)
    ENDDO

    jsl = nslm
    DO ji=1, kjpindex
       excess = MAX(mcr(njsc(ji))-mc(ji,jsl,ins),zero)
       mc(ji,jsl,ins) = mc(ji,jsl,ins) + excess
       mc(ji,jsl-1,ins) = mc(ji,jsl-1,ins) - excess * &
            &  dz(jsl,ins)/(dz(jsl-1,ins)+dz(jsl,ins))
    ENDDO

    DO jsl = nslm-1,2,-1
       DO ji=1, kjpindex
          excess = MAX(mcr(njsc(ji))-mc(ji,jsl,ins),zero)
          mc(ji,jsl,ins) = mc(ji,jsl,ins) + excess
          mc(ji,jsl-1,ins) = mc(ji,jsl-1,ins) - excess * &
               &  (dz(jsl,ins)+dz(jsl+1,ins))/(dz(jsl-1,ins)+dz(jsl,ins))
       ENDDO
    ENDDO

    DO ji=1, kjpindex
       excess = mask_soiltile(ji,ins) * MAX(mcr(njsc(ji))-mc(ji,1,ins),zero)
       mc(ji,1,ins) = mc(ji,1,ins) + excess
       
       ! Keep the value in case excess is still positive (due to big change in evapot)
       under_mcr(ji) = excess * dz(2,ins)/2                
    ENDDO

    ! We just get sure that mc remains at 0 where soiltile=0
    DO jsl = 1, nslm
       DO ji=1, kjpindex
          mc(ji,jsl,ins) = mask_soiltile(ji,ins) * mc(ji,jsl,ins)
       ENDDO
    ENDDO
    
    RETURN
  END SUBROUTINE hydrol_soil_smooth

  SUBROUTINE hydrol_soil_flux(kjpindex,ins,mcint,returnflow_soil)
    !
    !**** *hydrol_soil_flux* - 
    !**** *routine that computes the hydrological fluxes between the different soil layer.
    !
    INTEGER(i_std), INTENT(in)                         :: kjpindex        !! Domain size
    INTEGER(i_std), INTENT(in)                         :: ins             !! index of soil type
    REAL(r_std), DIMENSION (kjpindex,nslm), INTENT(in) :: mcint           !! mc values at the beginning of the time step
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)      :: returnflow_soil !! returnflow
    ! local 
    INTEGER(i_std) :: jsl,ji
    REAL(r_std), DIMENSION(kjpindex)                        :: temp
    
    !- Compute the flux at every level from bottom to top (using mc and sink values)
    DO ji = 1, kjpindex
       !- First initialize qflux from the bottom, with dr_ns
       jsl = nslm
       qflux(ji,jsl,ins) = dr_ns(ji,ins) - returnflow_soil(ji)
       
       jsl = nslm-1
       qflux(ji,jsl,ins) = qflux(ji,jsl+1,ins) & 
            &  + (mc(ji,jsl,ins)-mcint(ji,jsl) &
            &  + trois*mc(ji,jsl+1,ins) - trois*mcint(ji,jsl+1)) &
            &  * (dz(jsl+1,ins)/huit) &
            &  + rootsink(ji,jsl+1,ins) 
    ENDDO
       
    DO jsl = nslm-2,1,-1
       DO ji = 1, kjpindex
          qflux(ji,jsl,ins) = qflux(ji,jsl+1,ins) & 
               &  + (mc(ji,jsl,ins)-mcint(ji,jsl) &
               &  + trois*mc(ji,jsl+1,ins) - trois*mcint(ji,jsl+1)) &
               &  * (dz(jsl+1,ins)/huit) &
               &  + rootsink(ji,jsl+1,ins) &
               &  + (dz(jsl+2,ins)/huit) &
               &  * (trois*mc(ji,jsl+1,ins) - trois*mcint(ji,jsl+1) &
               &  + mc(ji,jsl+2,ins)-mcint(ji,jsl+2))
       END DO
    ENDDO
       
    DO ji = 1, kjpindex
       temp(ji) =  qflux(ji,1,ins) + (dz(2,ins)/huit) &
            &  * (trois* (mc(ji,1,ins)-mcint(ji,1)) + (mc(ji,2,ins)-mcint(ji,2))) &
            &  + rootsink(ji,1,ins)
    ENDDO

    DO ji = 1, kjpindex
       IF (ABS(qflux00(ji,ins)-temp(ji)).GT. deux*min_sechiba) THEN
          WRITE (numout,*) 'Problem in the water balance, qflux computation', qflux00(ji,ins),temp(ji)
          WRITE (numout,*) 'returnflow_soil', returnflow_soil(ji)
          WRITE(numout,*) 'ji', ji, 'jsl',jsl,'ins',ins
          WRITE(numout,*) 'mcint', mcint(ji,:)
          WRITE(numout,*) 'mc', mc(ji,:,ins)
          WRITE (numout,*) 'rootsink', rootsink(ji,1,ins)
          waterbal_error=.TRUE.
          CALL ipslerr(3, 'hydrol_soil_flux', 'We will STOP after hydrol_soil_flux.',&
               & 'Problem in the water balance, qflux computation','')
       ENDIF
    ENDDO

    RETURN
  END SUBROUTINE hydrol_soil_flux

  SUBROUTINE hydrol_soil_tridiag(kjpindex,ins)

    !- solves a set of linear equations which has  a tridiagonal
    !-  coefficient matrix.

    !- arguments
    INTEGER(i_std), INTENT(in)                         :: kjpindex        !! Domain size
    INTEGER(i_std), INTENT(in)                         :: ins             !! number of soil type

    !  -- variables locales

    INTEGER(i_std)                      ::  ji,jt,jsl
    REAL(r_std), DIMENSION(kjpindex)                             :: bet

    DO ji = 1, kjpindex
       IF (resolv(ji)) THEN
          bet(ji) = tmat(ji,1,2)
          mc(ji,1,ins) = rhs(ji,1)/bet(ji)
       ENDIF
    ENDDO

    DO jsl = 2,nslm
       DO ji = 1, kjpindex
          
          IF (resolv(ji)) THEN

             gam(ji,jsl) = tmat(ji,jsl-1,3)/bet(ji)
             bet(ji) = tmat(ji,jsl,2) - tmat(ji,jsl,1)*gam(ji,jsl)
             mc(ji,jsl,ins) = (rhs(ji,jsl)-tmat(ji,jsl,1)*mc(ji,jsl-1,ins))/bet(ji)
          ENDIF

       ENDDO
    ENDDO

    DO ji = 1, kjpindex
       IF (resolv(ji)) THEN
          DO jsl = nslm-1,1,-1
             mc(ji,jsl,ins) = mc(ji,jsl,ins) - gam(ji,jsl+1)*mc(ji,jsl+1,ins)
          ENDDO
       ENDIF
    ENDDO
    RETURN
  END SUBROUTINE hydrol_soil_tridiag
      
  SUBROUTINE hydrol_soil_coef(kjpindex,ins,njsc)

    !
    !**** *hydrol_soil_coef* - 
    !**** *computes coef for the linearised hydraulic conductivity k_lin=a_lin mc_lin+b_lin
    !**** *and the linearised diffusivity d_lin
    !
    IMPLICIT NONE
    !
    INTEGER(i_std), INTENT(in)                        :: kjpindex         !! Domain size
    INTEGER(i_std), INTENT(in)                        :: ins              ! index of soil type
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: njsc            !! Indeces of the soiltype 
    ! local 
    INTEGER(i_std) :: jsl,ji,i

    !-first, we identify the interval i in which the current value of mc is located
    !-then, we give the values of the linearized parameters to compute 
    ! conductivity and diffusivity as K=a*mc+b and d

    DO jsl=1,nslm
       DO ji=1,kjpindex 
          i= MAX(MIN(INT((imax-imin)*(mc(ji,jsl,ins)-mcr(njsc(ji))) &
               &         / (mcs(njsc(ji))-mcr(njsc(ji))))+imin , imax-1), imin)
          a(ji,jsl) = a_lin(i,jsl,njsc(ji)) * kfact_root(ji,jsl,ins)
          b(ji,jsl) = b_lin(i,jsl,njsc(ji)) * kfact_root(ji,jsl,ins)
          d(ji,jsl) = d_lin(i,jsl,njsc(ji)) * kfact_root(ji,jsl,ins)
          k(ji,jsl) = MAX(k_lin(imin+1,jsl,njsc(ji)), &
               & a_lin(i,jsl,njsc(ji)) * mc(ji,jsl,ins) + b_lin(i,jsl,njsc(ji))) 
       ENDDO ! loop on grid
    ENDDO

    RETURN
  END SUBROUTINE hydrol_soil_coef

  SUBROUTINE hydrol_soil_setup(kjpindex,ins,dtradia)

    !
    !**** *hydrol_soil_setup* - 
    !**** *routine that computes the matrix coef for dublin model.

    IMPLICIT NONE
    !
    REAL(r_std), INTENT (in)                           :: dtradia          !! Time step in seconds
    ! parameters
    INTEGER(i_std), INTENT(in)                        :: kjpindex         !! Domain size
    INTEGER(i_std), INTENT(in)                        :: ins              ! index of soil type
    ! local 
    INTEGER(i_std) :: jsl,ji,i
    REAL(r_std), DIMENSION (nslm)      :: temp0, temp5, temp6, temp7
    REAL(r_std)                        :: temp3, temp4

    !-we compute tridiag matrix coefficients (LEFT and RIGHT) 
    ! of the system to solve [LEFT]*mc_{t+1}=[RIGHT]*mc{t}+[add terms]: 
    ! e(nslm),f(nslm),g1(nslm) for the [left] vector
    ! and ep(nslm),fp(nslm),gp(nslm) for the [right] vector

    ! w_time=1 (in constantes_soil) indicates implicit computation for diffusion 
    temp3 = w_time*(dtradia/one_day)/deux
    temp4 = (un-w_time)*(dtradia/one_day)/deux

    !- coefficient for first layer
    DO ji = 1, kjpindex
       e(ji,1) = zero 
       f(ji,1) = trois * dz(2,ins)/huit  + temp3 &
            & * deux*SQRT(d(ji,1)*d(ji,2))/(dz(2,ins))
       g1(ji,1) = dz(2,ins)/(huit)       - temp3 &
            & * deux*SQRT(d(ji,1)*d(ji,2))/(dz(2,ins))
       ep(ji,1) = zero 
       fp(ji,1) = trois * dz(2,ins)/huit - temp4 &
            & * deux*SQRT(d(ji,1)*d(ji,2))/(dz(2,ins))
       gp(ji,1) = dz(2,ins)/(huit)       + temp4 &
            & * deux*SQRT(d(ji,1)*d(ji,2))/(dz(2,ins))
    ENDDO
    
    !- coefficient for medium layers
    DO jsl = 2, nslm-1
       DO ji = 1, kjpindex
          e(ji,jsl) = dz(jsl,ins)/(huit)                        - temp3 &
               & * deux*SQRT(d(ji,jsl)*d(ji,jsl-1))/(dz(jsl,ins))
          
          f(ji,jsl) = trois * (dz(jsl,ins)+dz(jsl+1,ins))/huit  + temp3 &
               & * ( deux*SQRT(d(ji,jsl)*d(ji,jsl-1))/(dz(jsl,ins)) + &
               & deux*SQRT(d(ji,jsl)*d(ji,jsl+1))/(dz(jsl+1,ins)) )
          
          g1(ji,jsl) = dz(jsl+1,ins)/(huit)                     - temp3 &
               & * deux*SQRT(d(ji,jsl)*d(ji,jsl+1))/(dz(jsl+1,ins)) 
          
          ep(ji,jsl) = dz(jsl,ins)/(huit)                       + temp4 & 
               & *  deux*SQRT(d(ji,jsl)*d(ji,jsl-1))/(dz(jsl,ins))
          
          fp(ji,jsl) = trois * (dz(jsl,ins)+dz(jsl+1,ins))/huit - temp4 &
               & * ( deux*SQRT(d(ji,jsl)*d(ji,jsl-1))/(dz(jsl,ins)) + &
               & deux*SQRT(d(ji,jsl)*d(ji,jsl+1))/(dz(jsl+1,ins)) )
          
          gp(ji,jsl) = dz(jsl+1,ins)/(huit)                     + temp4 &
               & * deux*SQRT(d(ji,jsl)*d(ji,jsl+1))/(dz(jsl+1,ins)) 
       ENDDO
    ENDDO
    
    !- coefficient for last layer
    DO ji = 1, kjpindex
       e(ji,nslm) = dz(nslm,ins)/(huit)        - temp3 &
            & * deux*SQRT(d(ji,nslm)*d(ji,nslm-1)) / (dz(nslm,ins)) 
       f(ji,nslm) = trois * dz(nslm,ins)/huit  + temp3 &
            & * deux*SQRT(d(ji,nslm)*d(ji,nslm-1)) / (dz(nslm,ins)) 
       g1(ji,nslm) = zero
       ep(ji,nslm) = dz(nslm,ins)/(huit)       + temp4 & 
            & * deux*SQRT(d(ji,nslm)*d(ji,nslm-1)) / (dz(nslm,ins)) 
       fp(ji,nslm) = trois * dz(nslm,ins)/huit - temp4 &
            & * deux*SQRT(d(ji,nslm)*d(ji,nslm-1)) / (dz(nslm,ins)) 
       gp(ji,nslm) = zero 
    ENDDO
    
    RETURN
  END SUBROUTINE hydrol_soil_setup

  !!! fait la connexion entre l'hydrologie et sechiba :
  !!! cherche les variables sechiba pour l'hydrologie
  !!! "transforme" ces variables
  SUBROUTINE hydrol_split_soil (kjpindex, veget, soiltile, vevapnu, transpir, humrel,evap_bare_lim)
    ! 
    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex
    REAL(r_std), DIMENSION (kjpindex, nvm), INTENT(in)       :: veget            !! Vegetation map 
    REAL(r_std), DIMENSION (kjpindex,nstm), INTENT (in)      :: soiltile         !! Map of soil types
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)           :: vevapnu          !! Bare soil evaporation
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)       :: transpir         !! Transpiration
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)       :: humrel           !! Relative humidity
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)           :: evap_bare_lim   !!   
    !
    ! local declaration
    !
    INTEGER(i_std)                               :: ji, jv, jsl, jst
    REAL(r_std), Dimension (kjpindex)             :: vevapnu_old
    REAL(r_std), Dimension (kjpindex)             :: tmp_check1
    REAL(r_std), Dimension (kjpindex)             :: tmp_check2
    REAL(r_std), DIMENSION (kjpindex,nstm)        :: tmp_check3
    REAL(r_std)                                   :: test
    !
    !
    ! split 2d variables into 3d variables, per soil type
    !
    precisol_ns(:,:)=zero
    DO jv=1,nvm
       DO jst=1,nstm
          DO ji=1,kjpindex
             IF(veget(ji,jv).GT.min_sechiba) THEN
                precisol_ns(ji,jst)=precisol_ns(ji,jst)+precisol(ji,jv)* &
                     & corr_veg_soil(ji,jv,jst) /vegtot(ji) / veget(ji,jv)
             ENDIF
          END DO
       END DO
    END DO
    !
    !
    !
    vevapnu_old(:)=zero
    DO jst=1,nstm
       DO ji=1,kjpindex
          vevapnu_old(ji)=vevapnu_old(ji)+ &
               & ae_ns(ji,jst)*soiltile(ji,jst)*vegtot(ji)
       END DO
    END DO
    !
    !
    !
    DO jst=1,nstm
       DO ji=1,kjpindex
          IF (vevapnu_old(ji).GT.min_sechiba) THEN   
             IF(evap_bare_lim(ji).GT.min_sechiba) THEN       
                ae_ns(ji,jst) = vevapnu(ji) * evap_bare_lim_ns(ji,jst)/evap_bare_lim(ji)
             ELSE
                ae_ns(ji,jst)=ae_ns(ji,jst) * vevapnu(ji)/vevapnu_old(ji)
             ENDIF
          ELSEIF(frac_bare_ns(ji,jst).GT.min_sechiba) THEN
             IF(evap_bare_lim(ji).GT.min_sechiba) THEN  
                ae_ns(ji,jst) = vevapnu(ji) * evap_bare_lim_ns(ji,jst)/evap_bare_lim(ji)
             ELSE
!!$                WRITE(numout,*) "ji,jst ",ji,jst," frac_bare_ns,tot_frac_bare",&
!!$                     frac_bare_ns(ji,jst),tot_frac_bare(ji)
                ae_ns(ji,jst) = vevapnu(ji) * frac_bare_ns(ji,jst)/tot_frac_bare(ji)
             ENDIF
!!$          ELSE
!!$                WRITE(numout,*) "Rien a faire pour ji,jst : ",ji,jst 
          ENDIF
          precisol_ns(ji,jst)=precisol_ns(ji,jst)+MAX(-ae_ns(ji,jst),zero)
       END DO
    END DO

    tr_ns(:,:)=zero
    DO jv=1,nvm
       DO jst=1,nstm
          DO ji=1,kjpindex
             IF (humrel(ji,jv).GT.min_sechiba) THEN 
                tr_ns(ji,jst)=tr_ns(ji,jst)+ cvs_over_veg(ji,jv,jst)*humrelv(ji,jv,jst)* & 
                     & transpir(ji,jv)/humrel(ji,jv)
             ENDIF
          END DO
       END DO
    END DO

    rootsink(:,:,:)=zero
    DO jv=1,nvm
       DO jsl=1,nslm
          DO jst=1,nstm
             DO ji=1,kjpindex
                IF (humrel(ji,jv).GT.min_sechiba) THEN 
                   rootsink(ji,jsl,jst) = rootsink(ji,jsl,jst) &
                        & + cvs_over_veg(ji,jv,jst)* (transpir(ji,jv)*us(ji,jv,jst,jsl))/ &
                        & humrel(ji,jv)
                END IF
             END DO
          END DO
       END DO
    END DO

    IF(check_cwrr) THEN
       DO jsl=1,nslm
          DO jst=1,nstm
             DO ji=1,kjpindex
                IF(mc(ji,jsl,jst).LT.-0.05) THEN
!                IF(kjit_.LT.2.OR.kjit_.GT.1484) THEN
                   WRITE(numout,*) 'CWRR split-----------------------------------------------'
                   WRITE(numout,*) 'ji,jst,jsl',ji,jst,jsl
                   WRITE(numout,*) 'mc',mc(ji,jsl,jst)
                   WRITE(numout,*) 'rootsink,us',rootsink(ji,:,jst),us(ji,:,jst,jsl)
                   WRITE(numout,*) 'corr_veg_soil',corr_veg_soil(ji,:,jst)
                   WRITE(numout,*) 'transpir',transpir(ji,:)
                   WRITE(numout,*) 'veget',veget(ji,:)
                   WRITE(numout,*) 'cvs_over_veg',cvs_over_veg(ji,:,jst)
                   WRITE(numout,*) 'humrel',humrel(ji,:)
                   WRITE(numout,*) 'humrelv (pour ce jst)',humrelv(ji,:,jst)
                   WRITE(numout,*) 'ae_ns',ae_ns(ji,jst)
                   WRITE(numout,*) 'tr_ns',tr_ns(ji,jst)
                   WRITE(numout,*) 'vevapnuold',vevapnu_old(ji)
                ENDIF
             END DO
          END DO
       END DO
    ENDIF


    ! Now we check if the deconvolution is correct and conserves the fluxes:

    IF (check_cwrr) THEN


       tmp_check1(:)=zero
       tmp_check2(:)=zero  

       ! First we check the precisol and evapnu

       DO jst=1,nstm
          DO ji=1,kjpindex
             tmp_check1(ji)=tmp_check1(ji) + &
                  & (precisol_ns(ji,jst)-MAX(-ae_ns(ji,jst),zero))* &
                  & soiltile(ji,jst)*vegtot(ji)
          END DO
       END DO

       DO jv=1,nvm
          DO ji=1,kjpindex
             tmp_check2(ji)=tmp_check2(ji) + precisol(ji,jv)
!!$             tmp_check2(ji)=tmp_check2(ji) + precisol(ji,jv)*vegtot(ji)
          END DO
       END DO


       DO ji=1,kjpindex   

          IF(ABS(tmp_check1(ji)- tmp_check2(ji)).GT.allowed_err) THEN
             WRITE(numout,*) 'PRECISOL SPLIT FALSE:ji=',ji,tmp_check1(ji),tmp_check2(ji)
             WRITE(numout,*) 'vegtot',vegtot(ji)

             DO jv=1,nvm
                WRITE(numout,*) 'jv,veget, precisol',jv,veget(ji,jv),precisol(ji,jv)
                DO jst=1,nstm 
                  WRITE(numout,*) 'corr_veg_soil:jst',jst,corr_veg_soil(ji,jv,jst)
                END DO
             END DO

             DO jst=1,nstm
                WRITE(numout,*) 'jst,precisol_ns',jst,precisol_ns(ji,jst)
                WRITE(numout,*) 'soiltile', soiltile(ji,jst)
             END DO
             waterbal_error=.TRUE.
             CALL ipslerr(2, 'hydrol_split_soil', 'We will STOP after hydrol_split_soil.',&
                  & 'check_CWRR','PRECISOL SPLIT FALSE')
          ENDIF

       END DO


       tmp_check1(:)=zero

       DO jst=1,nstm
          DO ji=1,kjpindex
             tmp_check1(ji)=tmp_check1(ji) + ae_ns(ji,jst)* &
                  & soiltile(ji,jst)*vegtot(ji)
          END DO
       END DO

       DO ji=1,kjpindex   

          IF(ABS(tmp_check1(ji)- vevapnu(ji)).GT.allowed_err) THEN
             WRITE(numout,*) 'VEVAPNU SPLIT FALSE:ji, Sum(ae_ns), vevapnu =',ji,tmp_check1(ji),vevapnu(ji)
             WRITE(numout,*) 'ae_ns',ae_ns(ji,:)
             WRITE(numout,*) 'vegtot',vegtot(ji)
             WRITE(numout,*) 'evap_bare_lim, evap_bare_lim_ns',evap_bare_lim(ji), evap_bare_lim_ns(ji,:)
             WRITE(numout,*) 'tot_frac_bare,frac_bare_ns',tot_frac_bare(ji),frac_bare_ns(ji,:)
             WRITE(numout,*) 'vevapnu_old',vevapnu_old(ji)
             DO jst=1,nstm
                WRITE(numout,*) 'jst,ae_ns',jst,ae_ns(ji,jst)
                WRITE(numout,*) 'soiltile', soiltile(ji,jst)
                WRITE(numout,*) 'veget_exist/soiltile', veget(ji,:)/vegtot(ji)/soiltile(ji,jst)
                WRITE(numout,*) "corr_veg_soil",corr_veg_soil(ji,:,jst)
             END DO
             waterbal_error=.TRUE.
             CALL ipslerr(2, 'hydrol_split_soil', 'We will STOP after hydrol_split_soil.',&
                  & 'check_CWRR','VEVAPNU SPLIT FALSE')
          ENDIF
       ENDDO

       ! Second we check the transpiration and root sink

       tmp_check1(:)=zero
       tmp_check2(:)=zero  


       DO jst=1,nstm
          DO ji=1,kjpindex
             tmp_check1(ji)=tmp_check1(ji) + tr_ns(ji,jst)* &
                  & soiltile(ji,jst)*vegtot(ji)
          END DO
       END DO

       DO jv=1,nvm
          DO ji=1,kjpindex
             tmp_check2(ji)=tmp_check2(ji) + transpir(ji,jv)
          END DO
       END DO

       DO ji=1,kjpindex   

          IF(ABS(tmp_check1(ji)- tmp_check2(ji)).GT.allowed_err) THEN
             WRITE(numout,*) 'TRANSPIR SPLIT FALSE:ji=',ji,tmp_check1(ji),tmp_check2(ji)
             WRITE(numout,*) 'vegtot',vegtot(ji)

             DO jv=1,nvm
                WRITE(numout,*) 'jv,veget, transpir',jv,veget(ji,jv),transpir(ji,jv)
                DO jst=1,nstm
                   WRITE(numout,*) 'corr_veg_soil:ji,jv,jst',ji,jv,jst,corr_veg_soil(ji,jv,jst)
                END DO
             END DO

             DO jst=1,nstm
                WRITE(numout,*) 'jst,tr_ns',jst,tr_ns(ji,jst)
                WRITE(numout,*) 'soiltile', soiltile(ji,jst)
             END DO
             waterbal_error=.TRUE.
             CALL ipslerr(2, 'hydrol_split_soil', 'We will STOP after hydrol_split_soil.',&
                  & 'check_CWRR','TRANSPIR SPLIT FALSE')
          ENDIF

       END DO


       tmp_check3(:,:)=zero

       DO jst=1,nstm
          DO jsl=1,nslm
             DO ji=1,kjpindex
                tmp_check3(ji,jst)=tmp_check3(ji,jst) + rootsink(ji,jsl,jst)
             END DO
          END DO
       ENDDO

       DO jst=1,nstm
          DO ji=1,kjpindex
             IF(ABS(tmp_check3(ji,jst)- tr_ns(ji,jst)).GT.allowed_err) THEN
                WRITE(numout,*) 'ROOTSINK SPLIT FALSE:ji,jst=', ji,jst,&
                     & tmp_check3(ji,jst),tr_ns(ji,jst)
                WRITE(numout,*) 'HUMREL(jv=1:13)',humrel(ji,:)
                WRITE(numout,*) 'TRANSPIR',transpir(ji,:)
                DO jv=1,nvm 
                   WRITE(numout,*) 'jv=',jv,'us=',us(ji,jv,jst,:)
                ENDDO
                waterbal_error=.TRUE.
                CALL ipslerr(2, 'hydrol_split_soil', 'We will STOP after hydrol_split_soil.',&
                  & 'check_CWRR','ROOTSINK SPLIT FALSE')
             ENDIF
          END DO
       END DO

    ENDIF

    RETURN 

  END SUBROUTINE hydrol_split_soil

  SUBROUTINE hydrol_diag_soil (kjpindex, veget, soiltile, njsc, runoff, drainage, &
       & evap_bare_lim, evapot, vevapnu, returnflow, reinfiltration, irrigation, &
       & shumdiag, k_litt, litterhumdiag, humrel, vegstress, drysoil_frac, tot_melt)
    ! 
    ! interface description
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex 
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)       :: veget           !! Map of vegetation types
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)        :: njsc            !! Indeces of the soiltype 
    REAL(r_std), DIMENSION (kjpindex,nstm), INTENT (in)      :: soiltile        !! Map of soil types
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: drysoil_frac    !! Function of litter wetness
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: runoff          !! complete runoff
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: drainage        !! Drainage
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)           :: evap_bare_lim   !! 
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: evapot          !! 
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: vevapnu  
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: returnflow       !! Water returning to the deep reservoir
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: reinfiltration   !! Water returning to the top of the soil
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: irrigation      !! Water from irrigation
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: tot_melt
    REAL(r_std),DIMENSION (kjpindex,nbdl), INTENT (out)      :: shumdiag        !! relative soil moisture
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: k_litt          !! litter cond.
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: litterhumdiag   !! litter humidity
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)       :: humrel          !! Relative humidity
    REAL(r_std), DIMENSION (kjpindex, nvm), INTENT(out)    :: vegstress       !! Veg. moisture stress (only for vegetation growth)
    !
    ! local declaration
    !
    INTEGER(i_std)                                :: ji, jv, jsl, jst, i
    REAL(r_std), DIMENSION (kjpindex)                                    :: mask_vegtot
    !
    ! Put the prognostics variables of soil to zero if soiltype is zero

    DO jst=1,nstm 

       DO ji=1,kjpindex

!          IF(soiltile(ji,jst).EQ.zero) THEN

             ae_ns(ji,jst) = ae_ns(ji,jst) * mask_soiltile(ji,jst)
             dr_ns(ji,jst) = dr_ns(ji,jst) * mask_soiltile(ji,jst)
             ru_ns(ji,jst) = ru_ns(ji,jst) * mask_soiltile(ji,jst)
             tmc(ji,jst) =  tmc(ji,jst) * mask_soiltile(ji,jst)

             DO jv=1,nvm
                humrelv(ji,jv,jst) = humrelv(ji,jv,jst) * mask_soiltile(ji,jst)
                DO jsl=1,nslm
                   us(ji,jv,jst,jsl) = us(ji,jv,jst,jsl)  * mask_soiltile(ji,jst)
                END DO
             END DO

             DO jsl=1,nslm          
                mc(ji,jsl,jst) = mc(ji,jsl,jst)  * mask_soiltile(ji,jst)
             END DO

!          ENDIF

       END DO
    END DO


    runoff(:) = zero
    drainage(:) = zero
    humtot(:) = zero
    evap_bare_lim(:) = zero
    evap_bare_lim_ns(:,:) = zero
    shumdiag(:,:)= zero
    k_litt(:) = zero
    litterhumdiag(:) = zero
    tmc_litt_mea(:) = zero
    soilmoist(:,:) = zero
    humrel(:,:) = zero
    vegstress(:,:) = zero
    swi(:) = zero
    !
    ! sum 3d variables in 2d variables with fraction of vegetation per soil type
    !


    DO ji = 1, kjpindex
       mask_vegtot(ji) = 0
       IF(vegtot(ji) .GT. min_sechiba) THEN
          mask_vegtot(ji) = 1
       ENDIF
    END DO

    DO ji = 1, kjpindex 
       ! Here we weight ae_ns by the fraction of bare evaporating soil. 
       ! This is given by frac_bare_ns, taking into account bare soil under vegetation
       ae_ns(ji,:) = mask_vegtot(ji) * ae_ns(ji,:) * frac_bare_ns(ji,:)
    END DO
    DO jst = 1, nstm
       DO ji = 1, kjpindex 
          drainage(ji) = mask_vegtot(ji) * (drainage(ji) + vegtot(ji)*soiltile(ji,jst) * dr_ns(ji,jst))
          runoff(ji) = mask_vegtot(ji) *  (runoff(ji) +   vegtot(ji)*soiltile(ji,jst) * ru_ns(ji,jst)) &
               & + (1 - mask_vegtot(ji)) * (tot_melt(ji) + irrigation(ji) + returnflow(ji) + reinfiltration(ji))
          humtot(ji) = mask_vegtot(ji) * (humtot(ji) + soiltile(ji,jst) * tmc(ji,jst)) 
       END DO
    END DO

    DO jst=1,nstm
       DO ji=1,kjpindex
          IF ((evapot(ji).GT.min_sechiba) .AND. &
               & (tmc_litter(ji,jst).GT.(tmc_litter_wilt(ji,jst)))) THEN
             evap_bare_lim_ns(ji,jst) = ae_ns(ji,jst) / evapot(ji)
          ELSEIF((evapot(ji).GT.min_sechiba).AND. &
               & (tmc_litter(ji,jst).GT.(tmc_litter_res(ji,jst)))) THEN
             evap_bare_lim_ns(ji,jst) =  (un/deux) * ae_ns(ji,jst) / evapot(ji)
          END IF
       END DO
    END DO

     DO ji = 1, kjpindex
         evap_bare_lim(ji) =   SUM(evap_bare_lim_ns(ji,:)*vegtot(ji)*soiltile(ji,:))
        IF(evap_bare_lim(ji).GT.un + min_sechiba) THEN
           print *,'CWRR DIAG EVAP_BARE_LIM TOO LARGE', ji, &
                & evap_bare_lim(ji),evap_bare_lim_ns(ji,:)
        ENDIF 
     ENDDO
    ! we add the excess of snow sublimation to vevapnu

    DO ji = 1,kjpindex
       vevapnu(ji) = vevapnu (ji) + subsinksoil(ji)*vegtot(ji)
    END DO

    DO jst=1,nstm
       DO jv=1,nvm
          DO ji=1,kjpindex
             IF(veget(ji,jv).GT.min_sechiba) THEN
                vegstress(ji,jv)=vegstress(ji,jv)+vegstressv(ji,jv,jst)*soiltile(ji,jst) &
                     & * corr_veg_soil(ji,jv,jst) *vegtot(ji)/veget(ji,jv)
                vegstress(ji,jv)= MAX(vegstress(ji,jv),zero)
             ENDIF
          END DO
       END DO
    END DO
    
    cvs_over_veg(:,:,:) = zero
    DO jv=1,nvm
       DO ji=1,kjpindex
          IF(veget(ji,jv).GT.min_sechiba) THEN
             DO jst=1,nstm
                cvs_over_veg(ji,jv,jst) = corr_veg_soil(ji,jv,jst)/vegtot(ji) / veget(ji,jv)
             ENDDO
          ENDIF
       END DO
    END DO

    DO jst=1,nstm
       DO jv=1,nvm
          DO ji=1,kjpindex
             humrel(ji,jv)=humrel(ji,jv)+humrelv(ji,jv,jst)*soiltile(ji,jst) &
                  & * cvs_over_veg(ji,jv,jst)*vegtot(ji)
             humrel(ji,jv)=MAX(humrel(ji,jv),zero)
          END DO
       END DO
    END DO

    DO jst=1,nstm
       DO ji=1,kjpindex
          ! We compute here a mean k for the 'litter' used for reinfiltration from floodplains of ponds
          i= MAX(MIN(INT((imax-imin)*(tmc_litter(ji,jst)-tmc_litter_res(ji,jst))&
               &         / (tmc_litter_sat(ji,jst)-tmc_litter_res(ji,jst)))+imin , imax-1), imin)
          k_litt(ji) = k_litt(ji) + soiltile(ji,jst) * SQRT(k_lin(i,1,njsc(ji))*ks(njsc(ji)))
       ENDDO
    ENDDO

    DO jst=1,nstm        

       DO ji=1,kjpindex
          litterhumdiag(ji) = litterhumdiag(ji) + &
               & soil_wet_litter(ji,jst) * soiltile(ji,jst)

          tmc_litt_mea(ji) = tmc_litt_mea(ji) + &
               & tmc_litter(ji,jst) * soiltile(ji,jst) 

       END DO


       DO jsl=1,nbdl
          DO ji=1,kjpindex
             shumdiag(ji,jsl)= shumdiag(ji,jsl) + soil_wet(ji,jsl,jst) * &
                  & ((mcs(njsc(ji))-mcw(njsc(ji)))/(mcf(njsc(ji))-mcw(njsc(ji)))) * &
                  & soiltile(ji,jst)
             soilmoist(ji,jsl)=soilmoist(ji,jsl)+mc(ji,jsl,jst)*soiltile(ji,jst)
!             shumdiag(ji,jsl) = MAX(MIN(shumdiag(ji,jsl), un), zero) 
          END DO
       END DO

    END DO
    
    ! First we compute swi (ALMIP requirement) - we assume here that dz is independant of jst (jst=1)
    jst=1
    DO ji=1,kjpindex
       swi(ji) = swi(ji) + shumdiag(ji,1) * (dz(2,jst))/(deux*dpu_max*mille)
       DO jsl=2,nslm-1 
          swi(ji) = swi(ji) + shumdiag(ji,jsl) * (dz(jsl,jst)+dz(jsl+1,jst))/(deux*dpu_max*mille)
       ENDDO
       jsl = nslm
       swi(ji) = swi(ji) + shumdiag(ji,jsl) * (dz(jsl,jst))/(deux*dpu_max*mille)
    ENDDO

    ! For stomate we need shumdiag to be bounded by 0 and 1
!!$    DO jsl=1,nbdl
!!$       DO ji=1,kjpindex
!!$          shumdiag(ji,jsl) = MAX(MIN(shumdiag(ji,jsl), un), zero) 
!!$       END DO
!!$    END DO
    shumdiag(:,:) = MIN(shumdiag(:,:), un)
    shumdiag(:,:) = MAX(shumdiag(:,:), zero) 

    DO ji=1,kjpindex
       drysoil_frac(ji) = un + MAX( MIN( (tmc_litt_dry_mea(ji) - tmc_litt_mea(ji)) / &
            & (tmc_litt_wet_mea(ji) - tmc_litt_dry_mea(ji)), zero), - un)
    END DO

  END SUBROUTINE hydrol_diag_soil

  !!
  !! This routines checks the water balance. First it gets the total
  !! amount of water and then it compares the increments with the fluxes.
  !! The computation is only done over the soil area as over glaciers (and lakes?)
  !! we do not have water conservation.
  !!
  !! This verification does not make much sense in REAL*4 as the precision is the same as some
  !! of the fluxes
  !!
  SUBROUTINE hydrol_waterbal (kjpindex, index, first_call, dtradia, veget, totfrac_nobio, &
       & qsintveg, snow, snow_nobio, precip_rain, precip_snow, returnflow, reinfiltration, irrigation, tot_melt, &
       & vevapwet, transpir, vevapnu, vevapsno, vevapflo, floodout, runoff, drainage)
    !
    !
    !
    INTEGER(i_std), INTENT (in)                        :: kjpindex     !! Domain size
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: index        !! Indeces of the points on the map
    LOGICAL, INTENT (in)                              :: first_call   !! At which time is this routine called ?
    REAL(r_std), INTENT (in)                           :: dtradia      !! Time step in seconds
    !
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget        !! Fraction of vegetation type 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: totfrac_nobio!! Total fraction of continental ice+lakes+...
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintveg     !! Water on vegetation due to interception
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: snow         !! Snow mass [Kg/m^2]
    REAL(r_std), DIMENSION (kjpindex,nnobio), INTENT(in)  :: snow_nobio    !!Ice water balance
    !
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: precip_rain  !! Rain precipitation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: precip_snow  !! Snow precipitation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: returnflow   !! Water to the bottom
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: reinfiltration !! Water to the top
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: irrigation   !! Water from irrigation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: tot_melt     !! Total melt
    !
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: vevapwet     !! Interception loss
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: transpir     !! Transpiration
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: vevapnu      !! Bare soil evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: vevapsno     !! Snow evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: vevapflo     !! Floodplains evaporation
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)       :: floodout     !! flow out of floodplains
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)       :: runoff       !! complete runoff
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: drainage     !! Drainage
    !
    !  LOCAL
    !
    INTEGER(i_std) :: ji
    REAL(r_std) :: watveg, delta_water
    !
    !
    !
    IF ( first_call ) THEN

       tot_water_beg(:) = zero

       DO ji = 1, kjpindex
          watveg = SUM(qsintveg(ji,:))
          tot_water_beg(ji) = humtot(ji)*vegtot(ji) + watveg + snow(ji)&
               & + SUM(snow_nobio(ji,:))
       ENDDO

       tot_water_end(:) = tot_water_beg(:)
       tot_flux(:) = zero

       RETURN

    ENDIF
    !
    !  Check the water balance
    !
    tot_water_end(:) = zero
    tot_flux(:) = zero
    !
    DO ji = 1, kjpindex
       !
       ! If the fraction of ice, lakes, etc. does not complement the vegetation fraction then we do not
       ! need to go any further
       !
!MM this test with DGVM ??
       IF ( ABS(un - (totfrac_nobio(ji) + vegtot(ji))) .GT. allowed_err ) THEN
          WRITE(numout,*) 'HYDROL problem in vegetation or frac_nobio on point ', ji
          WRITE(numout,*) 'totfrac_nobio : ', totfrac_nobio(ji)
          WRITE(numout,*) 'vegetation fraction : ', vegtot(ji)
          waterbal_error=.TRUE.
          CALL ipslerr(2, 'hydrol_waterbal', 'We will STOP after hydrol_waterbal.','','')
       ENDIF
    ENDDO

    IF ( .NOT. waterbal_error ) THEN
       DO ji = 1, kjpindex
          !
          watveg = SUM(qsintveg(ji,:))
          tot_water_end(ji) = humtot(ji)*vegtot(ji) + watveg + &
               & snow(ji) + SUM(snow_nobio(ji,:))
          !
          tot_flux(ji) =  precip_rain(ji) + precip_snow(ji) + irrigation (ji) - &
               & SUM(vevapwet(ji,:)) - SUM(transpir(ji,:)) - vevapnu(ji) - vevapsno(ji) - vevapflo(ji) + &
               & floodout(ji) - runoff(ji) - drainage(ji) + returnflow(ji) + reinfiltration(ji)
       ENDDO
       
       DO ji = 1, kjpindex
          !
          delta_water = tot_water_end(ji) - tot_water_beg(ji)
          !
          !
          !  Set some precision ! This is a wild guess and corresponds to what works on an IEEE machine
          !  under double precision (REAL*8).
          !
          !
          IF ( ABS(delta_water-tot_flux(ji)) .GT. deux*allowed_err ) THEN
             WRITE(numout,*) '------------------------------------------------------------------------- '
             WRITE(numout,*) 'HYDROL does not conserve water. The erroneous point is : ', ji
             WRITE(numout,*) 'Coord erroneous point', lalo(ji,:)
             WRITE(numout,*) 'The error in mm/s is :', (delta_water-tot_flux(ji))/dtradia, ' and in mm/dt : ', &
                  & delta_water-tot_flux(ji)
             WRITE(numout,*) 'delta_water : ', delta_water, ' tot_flux : ', tot_flux(ji)
             WRITE(numout,*) 'Actual and allowed error : ', ABS(delta_water-tot_flux(ji)), allowed_err
             WRITE(numout,*) 'vegtot : ', vegtot(ji)
             WRITE(numout,*) 'precip_rain : ', precip_rain(ji)
             WRITE(numout,*) 'precip_snow : ',  precip_snow(ji)
             WRITE(numout,*) 'Water from routing. Reinfiltration/returnflow/irrigation : ', reinfiltration(ji), &
                                                                               & returnflow(ji),irrigation(ji)
             WRITE(numout,*) 'Total water in soil humtot:',  humtot(ji)
             WRITE(numout,*) 'mc:' , mc(ji,:,:)
             WRITE(numout,*) 'Water on vegetation watveg:', watveg
             WRITE(numout,*) 'Snow mass snow:', snow(ji)
             WRITE(numout,*) 'Snow mass on ice snow_nobio:', SUM(snow_nobio(ji,:))
             WRITE(numout,*) 'Melt water tot_melt:', tot_melt(ji)
             WRITE(numout,*) 'evapwet : ', vevapwet(ji,:)
             WRITE(numout,*) 'transpir : ', transpir(ji,:)
             WRITE(numout,*) 'evapnu, evapsno, evapflo: ', vevapnu(ji), vevapsno(ji), vevapflo(ji)
             WRITE(numout,*) 'drainage,runoff,floodout : ', drainage(ji),runoff(ji),floodout(ji)
             waterbal_error=.TRUE.
             CALL ipslerr(2, 'hydrol_waterbal', 'We will STOP after hydrol_waterbal.','','')
          ENDIF
          !
       ENDDO
       !
       ! Transfer the total water amount at the end of the current timestep top the begining of the next one.
       !
       tot_water_beg = tot_water_end
       !
    ENDIF

  END SUBROUTINE hydrol_waterbal
  !
  !  This routine computes the changes in soil moisture and interception storage for the ALMA outputs
  !
  SUBROUTINE hydrol_alma (kjpindex, index, first_call, qsintveg, snow, snow_nobio, soilwet)
    !
    INTEGER(i_std), INTENT (in)                        :: kjpindex     !! Domain size
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)   :: index        !! Indeces of the points on the map
    LOGICAL, INTENT (in)                              :: first_call   !! At which time is this routine called ?
    !
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintveg     !! Water on vegetation due to interception
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: snow         !! Snow water equivalent
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: soilwet     !! Soil wetness
    REAL(r_std),DIMENSION (kjpindex,nnobio), INTENT (in) :: snow_nobio     !! Water balance on ice, lakes, .. [Kg/m^2]
    !
    !  LOCAL
    !
    INTEGER(i_std) :: ji
    REAL(r_std) :: watveg
    !
    !
    !
    IF ( first_call ) THEN

       tot_watveg_beg(:) = zero
       tot_watsoil_beg(:) = zero
       snow_beg(:)        = zero
       !
       DO ji = 1, kjpindex
          watveg = SUM(qsintveg(ji,:))
          tot_watveg_beg(ji) = watveg
          tot_watsoil_beg(ji) = humtot(ji)
          snow_beg(ji)        = snow(ji)+ SUM(snow_nobio(ji,:))
       ENDDO
       !
       tot_watveg_end(:) = tot_watveg_beg(:)
       tot_watsoil_end(:) = tot_watsoil_beg(:)
       snow_end(:)        = snow_beg(:)

       RETURN

    ENDIF
    !
    ! Calculate the values for the end of the time step
    !
    tot_watveg_end(:) = zero
    tot_watsoil_end(:) = zero
    snow_end(:) = zero
    delintercept(:) = zero
    delsoilmoist(:) = zero
    delswe(:) = zero
    !
    DO ji = 1, kjpindex
       watveg = SUM(qsintveg(ji,:))
       tot_watveg_end(ji) = watveg
       tot_watsoil_end(ji) = humtot(ji)
       snow_end(ji) = snow(ji)+ SUM(snow_nobio(ji,:))
       !
       delintercept(ji) = tot_watveg_end(ji) - tot_watveg_beg(ji)
       delsoilmoist(ji) = tot_watsoil_end(ji) - tot_watsoil_beg(ji)
       delswe(ji)       = snow_end(ji) - snow_beg(ji)
       !
       !
    ENDDO
    !
    !
    ! Transfer the total water amount at the end of the current timestep top the begining of the next one.
    !
    tot_watveg_beg = tot_watveg_end
    tot_watsoil_beg = tot_watsoil_end
    snow_beg(:) = snow_end(:)
    !
    DO ji = 1,kjpindex
       soilwet(ji) = tot_watsoil_end(ji) / mx_eau_var(ji)
    ENDDO
    !
  END SUBROUTINE hydrol_alma
  !
  !
END MODULE hydrol
